/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-bin-path.c - Path through binary tree.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include "flux-bin-path.h"
#include <string.h>         /* memcpy, memset */

#define DEBUG(x)

#define assert_types(o) flux_object_check (FLUX_OBJECT (o), FLUX_BIN_PATH_TYPE)

FluxBinPath *
flux_bin_path_new (void)
{
  FluxBinPath *bin_path;

  bin_path = g_new0 (FluxBinPath, 1);
  flux_bin_path_init (bin_path);

  return bin_path;
}

void
flux_bin_path_init (FluxBinPath *bin_path)
{
  flux_bool_array_init          (FLUX_BOOL_ARRAY (bin_path));
  flux_object_set_type          (FLUX_OBJECT (bin_path), FLUX_BIN_PATH_TYPE);
  flux_object_set_finalize_func (FLUX_OBJECT (bin_path), flux_bin_path_finalize);
}

void
flux_bin_path_finalize (FluxObject *object)
{
  flux_bool_array_finalize (object);
}

static guint
get_depth (FluxBinPath *bin_path)
{
  return flux_bool_array_get_size (FLUX_BOOL_ARRAY (bin_path));
}

guint
flux_bin_path_get_depth (FluxBinPath *bin_path)
{
  assert_types (bin_path);

  return get_depth (bin_path);
}

void
flux_bin_path_left (FluxBinPath *bin_path)
{
  assert_types (bin_path);

  flux_bool_array_append (FLUX_BOOL_ARRAY (bin_path), FALSE);
}

void
flux_bin_path_right (FluxBinPath *bin_path)
{
  assert_types (bin_path);

  flux_bool_array_append (FLUX_BOOL_ARRAY (bin_path), TRUE);
}

void
flux_bin_path_up (FluxBinPath *bin_path)
{
  FluxBoolArray *bool_array = FLUX_BOOL_ARRAY (bin_path);

  assert_types (bin_path);
  g_return_if_fail (get_depth (bin_path) > 0);

  flux_bool_array_remove (bool_array, flux_bool_array_get_size (bool_array) - 1);
}

gboolean
flux_bin_path_get_value (FluxBinPath *bin_path, guint depth)
{
  assert_types (bin_path);
  g_return_val_if_fail (depth < get_depth (bin_path), FALSE);

  return flux_bool_array_get_value (FLUX_BOOL_ARRAY (bin_path), depth);
}
