/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-bool-array.h - Packed array of booleans.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef FLUX_BOOL_ARRAY_H
#define FLUX_BOOL_ARRAY_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#define FLUX_BOOL_ARRAY(object) ((FluxBoolArray *) (object))

struct _FluxBoolArray
{
  FluxObject     object;

  guchar        *data;
  guint          size;
};

FluxBoolArray     *flux_bool_array_new             (void);
void               flux_bool_array_init            (FluxBoolArray *bool_array);
void               flux_bool_array_finalize        (FluxObject *bool_array);

guint              flux_bool_array_get_size        (FluxBoolArray *bool_array);
void               flux_bool_array_set_size        (FluxBoolArray *bool_array, guint len);

void               flux_bool_array_append          (FluxBoolArray *bool_array, gboolean val);
void               flux_bool_array_prepend         (FluxBoolArray *bool_array, gboolean val);
void               flux_bool_array_insert          (FluxBoolArray *bool_array, guint index, gboolean val);
void               flux_bool_array_remove          (FluxBoolArray *bool_array, guint index);

gboolean           flux_bool_array_get_value       (FluxBoolArray *bool_array, guint index);
void               flux_bool_array_set_value       (FluxBoolArray *bool_array, guint index, gboolean val);

void               flux_bool_array_peek_data       (FluxBoolArray *bool_array, gconstpointer *data,
                                                    guint *byte_len);
void               flux_bool_array_set_data        (FluxBoolArray *bool_array, gconstpointer src, guint len);

#endif  /* FLUX_BOOL_ARRAY_H */
