/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-cached-file.h - Simple, in-process wrapper cache for seekable fd.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_CACHED_FILE_H
#define _FLUX_CACHED_FILE_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#define FLUX_CACHED_FILE(object) ((FluxCachedFile *) object)

struct _FluxCachedFile
{
  FluxObject object;

  gint       fd;
  guint      actual_position;
  guint      expected_position;

  guint      elem_size;
  guint      elem_count;

  guint     *key;
  gpointer   data;

  gpointer   peek_buffer;

  gulong     hits;
  gulong     misses;
};

void            flux_cached_file_init           (FluxCachedFile *cached_file, gint fd,
                                                 guint elem_size, guint elem_count);
FluxCachedFile *flux_cached_file_new            (gint fd, guint elem_size, guint elem_count);
void            flux_cached_file_finalize       (FluxObject *cached_file);

gint            flux_cached_file_get_fd         (FluxCachedFile *cached_file);

guint           flux_cached_file_tell           (FluxCachedFile *cached_file);
gboolean        flux_cached_file_seek           (FluxCachedFile *cached_file, guint offset);

gconstpointer   flux_cached_file_peek           (FluxCachedFile *cached_file, guint len);
gboolean        flux_cached_file_read           (FluxCachedFile *cached_file, gpointer dest, guint len);
gboolean        flux_cached_file_write          (FluxCachedFile *cached_file, gconstpointer src, guint len);

#endif /* _FLUX_CACHED_FILE_H */
