/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-cipher.h - Generic encryption/decryption context.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_CIPHER_H
#define _FLUX_CIPHER_H

#include <gmp.h>

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#define FLUX_CIPHER(cip)  ((FluxCipher *) cip)

/* The cipher context structure. */

struct _FluxCipher
{
  FluxObject object;

  guint got_invitation    : 1,
        sent_confirmation : 1,
        got_confirmation  : 1;

  FluxObject *sym_key_encrypt;    /* Symmetric key used to encrypt this session */
  FluxObject *sym_key_decrypt;    /* Symmetric key used to decrypt this session */

  FluxObject *asym_key_encrypt;   /* Asymmetric key used to encrypt (her public) */
  FluxObject *asym_key_decrypt;   /* Asymmetric key used to decrypt (my secret) */

  FluxObject *asym_key_sign;      /* Asymmetric key used to sign (my secret) */
  FluxObject *asym_key_verify;    /* Asymmetric key used to verify (her public) */
};

/* --- Generic cipher functions --- */

FluxCipher *flux_cipher_new                 (void);
FluxCipher *flux_cipher_dup                 (FluxCipher *old);
void        flux_cipher_finalize            (FluxObject *cip);

void        flux_cipher_generate            (FluxCipher *cip);
void        flux_cipher_randomize           (FluxCipher *cip);

void        flux_cipher_export_invitation   (FluxCipher *cip, guchar **out, guint *len);
gboolean    flux_cipher_import_invitation   (FluxCipher *cip, const guchar *in, guint len);

void        flux_cipher_export_confirmation (FluxCipher *cip, guchar **out, guint *len);
gboolean    flux_cipher_import_confirmation (FluxCipher *cip, const guchar *in, guint len);

void        flux_cipher_encrypt_buffer      (FluxCipher *cip, guchar *buf, guint len);
void        flux_cipher_decrypt_buffer      (FluxCipher *cip, guchar *buf, guint len);

void        flux_cipher_encrypt_buffer_copy (FluxCipher *cip, guchar *out, const guchar *in, guint len);
void        flux_cipher_decrypt_buffer_copy (FluxCipher *cip, guchar *out, const guchar *in, guint len);

gboolean    flux_cipher_is_ready            (FluxCipher *cip);

#endif  /* _FLUX_CIPHER_H */
