/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-comm.h - N-ary tree-based protocol layer for FluxIO.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_COMM_H
#define _FLUX_COMM_H

#ifndef __FLUX_BUILD
#  include <flux/flux-types.h>
#  include <flux/flux-io.h>
#  include <flux/flux-tree.h>
#else
#  include "flux-types.h"
#  include "flux-io.h"
#  include "flux-tree.h"
#endif

#define FLUX_COMM(comm) ((FluxComm *) comm)

struct _FluxComm
{
  FluxObject object;

  FluxIO *io;

  FluxTree *queue_in, *queue_out;

  FluxTree *root_in, *root_out;
  FluxTree *node_in, *node_out;

  FluxCommSentFunc *sent_out_func;
  gpointer sent_out_func_data;

  FluxTree *sent_func_tree;

  gboolean recursing;
  gboolean do_auto_flush;

  guint32 block_in_size, block_out_size;
  guint32 block_in_bytes_left, block_out_bytes_left;
  guint32 node_in_bytes_left, node_out_bytes_left;
  guint32 block_in_level, block_out_level;  /* Breadth-first traversal */

  guint16 state_in : 2;  /* None, block header, node header, node data */
  guint16 state_out : 2;

  guint16 order_in : 1;
  guint16 order_out : 1;
  
  guint16 trans_in, trans_out;
  
  guint32 block_nodes_max, block_size_max, block_depth_max;
  guint16 trans_last;

  FluxCommDequeueFunc *dequeue_func;
  gpointer dequeue_func_data;
};

/* --- */

void      flux_comm_init           (FluxComm *comm, FluxIO *io);
FluxComm *flux_comm_new            (FluxIO *io);
void      flux_comm_finalize       (FluxObject *obj);

void      flux_comm_enqueue_full   (FluxComm *comm, FluxTree *tree, FluxCommOrder order,
                                    FluxCommSentFunc *func, gpointer func_data);
void      flux_comm_enqueue        (FluxComm *comm, FluxTree *tree);
FluxTree *flux_comm_dequeue        (FluxComm *comm);
void      flux_comm_flush          (FluxComm *comm);
void      flux_comm_reset          (FluxComm *comm);

FluxIO   *flux_comm_get_io         (FluxComm *comm);
void      flux_comm_set_dequeue_func (FluxComm *comm, FluxCommDequeueFunc *func, gpointer data);
void      flux_comm_set_limits     (FluxComm *c, gulong nodes, gulong size, gulong depth);
void      flux_comm_get_limits     (FluxComm *c, gulong *nodes, gulong *size, gulong *depth);
void      flux_comm_set_auto_flush (FluxComm *comm, gboolean auto_flush);

#endif /* _FLUX_COMM_H */
