/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-data-cache.h - Simple, flat cache for random access data.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_DATA_CACHE_H
#define _FLUX_DATA_CACHE_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#define FLUX_DATA_CACHE(object) ((FluxDataCache *) object)

struct _FluxDataCache
{
  FluxObject object;

  guint      elem_size;
  guint      elem_num;

  guint     *key;
  gpointer   data;

  gulong     hits;
  gulong     misses;
};

void           flux_data_cache_init           (FluxDataCache *cache, guint elem_size, guint elem_num);
FluxDataCache *flux_data_cache_new            (guint elem_size, guint elem_num);
void           flux_data_cache_finalize       (FluxObject *cache);

void           flux_data_cache_flush          (FluxDataCache *cache);
gconstpointer  flux_data_cache_insert         (FluxDataCache *cache, guint key, gconstpointer data);
gpointer       flux_data_cache_get_insert_ptr (FluxDataCache *cache, guint key);
gconstpointer  flux_data_cache_get_ptr        (FluxDataCache *cache, guint key);
gboolean       flux_data_cache_get_data       (FluxDataCache *cache, guint key, gpointer dest);
void           flux_data_cache_invalidate     (FluxDataCache *cache, guint key);

#endif /* _FLUX_DATA_CACHE_H */
