/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-db-header.h - DB file header.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef FLUX_DB_HEADER_H
#define FLUX_DB_HEADER_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#define FLUX_DB_HEADER_TYPE_SIZE 32
#define FLUX_DB_HEADER_USER_SIZE 32
#define FLUX_DB_HEADER_SIZE      (FLUX_DB_HEADER_TYPE_SIZE + sizeof (guint32) * 2 + sizeof (guint8) * 5 + \
                                 sizeof (guint16) + FLUX_DB_HEADER_USER_SIZE)

typedef struct
{
  gchar   type [FLUX_DB_HEADER_TYPE_SIZE];
  guint32 version;
  guint32 byte_order;
  guint8  int_size;
  guint8  long_size;
  guint8  short_size;
  guint8  float_size;
  guint8  double_size;
  guint16 pad_test_size;
  gchar   user [FLUX_DB_HEADER_USER_SIZE];
}
FluxDBHeader;

FluxDBHeader *flux_db_header_create (const gchar *type, guint32 version);
FluxDBHeader *flux_db_header_read   (gint fd);
void          flux_db_header_write  (FluxDBHeader *header, gint fd);
void          flux_db_header_free   (FluxDBHeader *header);

#endif  /* FLUX_DB_HEADER_H */
