/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-disk-chunk.h - Low-overhead same-size disk block allocator.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 *
 * Loosely modeled after flux-mem-chunk.
 */

#ifndef FLUX_DISK_CHUNK_H
#define FLUX_DISK_CHUNK_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-db-header.h"
# include "flux-data-cache.h"
# include "flux-storage.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-db-header.h>
# include <flux/flux-data-cache.h>
# include <flux/flux-storage.h>
#endif

#define FLUX_DISK_CHUNK(object) ((FluxDiskChunk *) (object))

typedef struct _DiskChunkFreeNode DiskChunkFreeNode;

struct _FluxDiskChunk
{
  FluxStorage    storage;

  gint           fd;
  guint          atomsize;  /* Size of each atom */
  guint          file_size;
  guint          free_size;

  FluxDataCache *data_cache;

  FluxItem       first_free_atom;

  GHashTable    *tag_table;
  gint           tag_arena_end;
};

FluxDiskChunk *flux_disk_chunk_new           (gint fd, guint atomsize);
gboolean       flux_disk_chunk_init          (FluxDiskChunk *disk_chunk, gint fd,
                                              guint atomsize);
void           flux_disk_chunk_finalize      (FluxObject *disk_chunk);

#endif  /* FLUX_DISK_CHUNK_H */
