/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-disk-heap.h - Low-overhead any-size disk allocator.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef FLUX_DISK_HEAP_H
#define FLUX_DISK_HEAP_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-db-header.h"
# include "flux-storage.h"
# include "flux-cached-file.h"
# include "flux-cursor.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-db-header.h>
# include <flux/flux-storage.h>
# include <flux/flux-cached-file.h>
# include <flux/flux-cursor.h>
#endif

#define FLUX_DISK_HEAP(object)        ((FluxDiskHeap *) (object))
#define FLUX_DISK_HEAP_CURSOR(object) ((FluxDiskHeapCursor *) (object))

struct _FluxDiskHeapCursor
{
  FluxCursor    cursor;

  FluxDiskHeap *disk_heap;

  FluxItem      current_ptr;
};

struct _FluxDiskHeap
{
  FluxStorage      storage;

  FluxCachedFile  *cached_file;
  guint            file_size;
  guint            free_size;

  FluxItem        *free_table;
};

FluxDiskHeap       *flux_disk_heap_new              (gint fd);
gboolean            flux_disk_heap_init             (FluxDiskHeap *disk_heap, gint fd);
void                flux_disk_heap_finalize         (FluxObject *disk_heap);

FluxDiskHeapCursor *flux_disk_heap_traverse         (FluxDiskHeap *disk_heap);

void                flux_disk_heap_cursor_init      (FluxDiskHeapCursor *disk_heap_cursor,
                                                     FluxDiskHeap *disk_heap);
void                flux_disk_heap_cursor_finalize  (FluxObject *object);

FluxItem            flux_disk_heap_cursor_get_pos   (FluxDiskHeapCursor *cursor);

#endif  /* FLUX_DISK_HEAP_H */
