/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-dual-fifo.h - Full-duplex byte FIFO with optional compression and ciphering.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_DUAL_FIFO_H
#define _FLUX_DUAL_FIFO_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-cipher.h"
# include "flux-fifo.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-cipher.h>
# include <flux/flux-fifo.h>
#endif

#define FLUX_DUAL_FIFO(dfifo) ((FluxDualFifo *) (dfifo))

struct _FluxDualFifo
{
  FluxObject     object;

  FluxFifo      *fifo [2];

  FluxReadHook  *read_hook;
  FluxWriteHook *write_hook;

  gpointer       read_hook_data;
  gpointer       write_hook_data;
};

/* --- */

FluxDualFifo *flux_dual_fifo_new             (gint buffers_min, gint buffers_max, gint buf_size);
void          flux_dual_fifo_init            (FluxDualFifo *dfifo,
                                              gint buffers_min, gint buffers_max, gint buf_size);
void          flux_dual_fifo_finalize        (FluxObject *dfifo);

void          flux_dual_fifo_write           (FluxDualFifo *dfifo, gconstpointer source, gint len);
gboolean      flux_dual_fifo_read            (FluxDualFifo *dfifo, gpointer dest, gint len);
void          flux_dual_fifo_flush_output    (FluxDualFifo *dfifo);

void          flux_dual_fifo_reset           (FluxDualFifo *dfifo);

FluxFifo     *flux_dual_fifo_get_write_fifo  (FluxDualFifo *dfifo);
FluxFifo     *flux_dual_fifo_get_read_fifo   (FluxDualFifo *dfifo);

gint          flux_dual_fifo_get_max_write   (FluxDualFifo *dfifo);
gint          flux_dual_fifo_get_max_read    (FluxDualFifo *dfifo);

void          flux_dual_fifo_set_compression (FluxDualFifo *dfifo, gint read_level, gint write_level);
void          flux_dual_fifo_set_cipher      (FluxDualFifo *dfifo, FluxCipher *cipher);

void          flux_dual_fifo_set_read_hook   (FluxDualFifo *dfifo, FluxReadHook *hook, gpointer data);
void          flux_dual_fifo_set_write_hook  (FluxDualFifo *dfifo, FluxWriteHook *hook, gpointer data);

#endif  /* _FLUX_DUAL_FIFO_H */
