/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-envoy.h - N-ary tree-based invocation layer for FluxIO.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson.
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_ENVOY_H
#define _FLUX_ENVOY_H

#ifndef __FLUX_BUILD
#  include <flux/flux-types.h>
#  include <flux/flux-comm.h>
#else
#  include "flux-types.h"
#  include "flux-comm.h"
#endif

#define FLUX_ENVOY(envoy) ((FluxEnvoy *) envoy)

struct _FluxEnvoy
{
  FluxComm comm;

  GList *assoc_list;
  guint next_id;
};

struct _FluxEnvoyDequeuer
{
  const gchar *token;
  FluxEnvoyDequeueFunc *func;
};

/* --- */

void       flux_envoy_init                      (FluxEnvoy *envoy, FluxIO *io);
FluxEnvoy *flux_envoy_new                       (FluxIO *io);
void       flux_envoy_finalize                  (FluxObject *obj);

guint      flux_envoy_add_dequeuer_before       (FluxEnvoy *envoy, const gchar *token,
                                                 FluxEnvoyDequeueFunc *func, gpointer data);
guint      flux_envoy_add_dequeuer_after        (FluxEnvoy *envoy, const gchar *token,
                                                 FluxEnvoyDequeueFunc *func, gpointer data);

void       flux_envoy_add_dequeuer_list_before  (FluxEnvoy *envoy, FluxEnvoyDequeuer *dequeuers,
                                                gpointer data);
void       flux_envoy_add_dequeuer_list_after   (FluxEnvoy *envoy, FluxEnvoyDequeuer *dequeuers,
                                                 gpointer data);

void       flux_envoy_remove_dequeuer_by_id     (FluxEnvoy *envoy, guint id);
void       flux_envoy_remove_dequeuers_by_token (FluxEnvoy *envoy, const gchar *token);
void       flux_envoy_remove_dequeuers_by_func  (FluxEnvoy *envoy,
                                                 FluxEnvoyDequeueFunc *func);
void       flux_envoy_remove_dequeuers_by_data  (FluxEnvoy *envoy, gpointer data);
void       flux_envoy_remove_dequeuers_by_token_and_func (FluxEnvoy *envoy, const gchar *token,
                                                          FluxEnvoyDequeueFunc *func);
void       flux_envoy_remove_dequeuer_by_all    (FluxEnvoy *envoy, const gchar *token,
                                                 FluxEnvoyDequeueFunc *func, gpointer data);

void       flux_envoy_remove_dequeuer_list      (FluxEnvoy *envoy, FluxEnvoyDequeuer *dequeuers);

#endif /* _FLUX_ENVOY_H */
