/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-ext-hash-table.h - Hash table with multiple values per key.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_EXT_HASH_TABLE_H
#define _FLUX_EXT_HASH_TABLE_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#include <glib.h>

#define FLUX_EXT_HASH_TABLE(object) ((FluxExtHashTable *) object)

struct _FluxExtHashTable
{
  FluxObject      object;

  GHashTable     *hash_table;
  GDestroyNotify  key_destroy_func;
  GDestroyNotify  value_destroy_func;
};

FluxExtHashTable *flux_ext_hash_table_new            (GHashFunc hash_func, GEqualFunc equal_func);
FluxExtHashTable *flux_ext_hash_table_new_full       (GHashFunc hash_func, GEqualFunc equal_func,
                                                      GDestroyNotify key_destroy_func,
                                                      GDestroyNotify value_destroy_func);
void              flux_ext_hash_table_init           (FluxExtHashTable *ext_hash_table,
                                                      GHashFunc hash_func, GEqualFunc equal_func,
                                                      GDestroyNotify key_destroy_func,
                                                      GDestroyNotify value_destroy_func);
void              flux_ext_hash_table_finalize       (FluxObject *object);

void              flux_ext_hash_table_insert         (FluxExtHashTable *ext_hash_table,
                                                      gpointer key, gpointer value);
guint             flux_ext_hash_table_size           (FluxExtHashTable *ext_hash_table);
GList            *flux_ext_hash_table_lookup         (FluxExtHashTable *ext_hash_table, gconstpointer key);
void              flux_ext_hash_table_foreach        (FluxExtHashTable *ext_hash_table,
                                                      GHFunc func, gpointer user_data);
gboolean          flux_ext_hash_table_remove_key     (FluxExtHashTable *ext_hash_table, gconstpointer key);
gboolean          flux_ext_hash_table_remove_value   (FluxExtHashTable *ext_hash_table,
                                                      gconstpointer key, gconstpointer value);
gboolean          flux_ext_hash_table_steal_key      (FluxExtHashTable *ext_hash_table, gconstpointer key);
gboolean          flux_ext_hash_table_steal_value    (FluxExtHashTable *ext_hash_table,
                                                      gconstpointer key, gconstpointer value);
guint             flux_ext_hash_table_foreach_remove (FluxExtHashTable *ext_hash_table,
                                                      GHRFunc func, gpointer user_data);
guint             flux_ext_hash_table_foreach_steal  (FluxExtHashTable *ext_hash_table,
                                                      GHRFunc func, gpointer user_data);

#endif /* _FLUX_EXT_HASH_TABLE_H */
