/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-ext-pat-tree.h - Extended on-disk indexing tree.
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef FLUX_EXT_PAT_TREE_H
#define FLUX_EXT_PAT_TREE_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-pat-tree.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-pat-tree.h>
#endif

#define FLUX_EXT_PAT_TREE(object) ((FluxExtPatTree *) (object))
#define FLUX_EXT_PAT_TREE_CURSOR(object) ((FluxExtPatTreeCursor *) (object))

struct _FluxExtPatTreeCursor
{
  FluxCursor         cursor;

  FluxExtPatTree    *ext_pat_tree;

  guint              traverse_values : 1;
  guint              random_order : 1;
  gpointer           key;
  guint              key_len;

  FluxPatTreeCursor *key_cursor;
  FluxPatTreeCursor *value_cursor;
};

struct _FluxExtPatTree
{
  FluxObject   object;

  guint        key_len;    /* Size of key data appended to each node */
  guint        value_len;  /* Size of value data appended to each node */

  FluxStorage *key_storage;
  FluxStorage *value_storage;

  FluxPatTree *key_tree;
  FluxPatTree *value_trees;
};

FluxExtPatTree       *flux_ext_pat_tree_new                (FluxStorage *key_storage, FluxStorage *value_storage,
                                                            const gchar *id,
                                                            guint key_len, guint value_len);
gboolean              flux_ext_pat_tree_init               (FluxExtPatTree *ext_pat_tree,
                                                            FluxStorage *key_storage, FluxStorage *value_storage,
                                                            const gchar *id, guint key_len, guint value_len);
void                  flux_ext_pat_tree_finalize           (FluxObject *ext_pat_tree);

gboolean              flux_ext_pat_tree_insert             (FluxExtPatTree *ext_pat_tree,
                                                            gconstpointer key, guint key_len,
                                                            gconstpointer value, guint value_len);
FluxExtPatTreeCursor *flux_ext_pat_tree_find               (FluxExtPatTree *ext_pat_tree,
                                                            gconstpointer key, guint key_len);
FluxExtPatTreeCursor *flux_ext_pat_tree_find_closest       (FluxExtPatTree *ext_pat_tree,
                                                            gconstpointer key, guint key_len);
FluxExtPatTreeCursor *flux_ext_pat_tree_find_with_mask     (FluxExtPatTree *ext_pat_tree,
                                                            gconstpointer key, gconstpointer key_mask,
                                                            guint key_len);
gboolean              flux_ext_pat_tree_remove_value       (FluxExtPatTree *ext_pat_tree,
                                                            gconstpointer key, guint key_len,
                                                            gconstpointer value, guint value_len);

gboolean              flux_ext_pat_tree_get_random         (FluxExtPatTree *ext_pat_tree,
                                                            gpointer *key, guint *key_len,
                                                            gpointer *value, guint *value_len);
gboolean              flux_ext_pat_tree_get_random_value   (FluxExtPatTree *ext_pat_tree,
                                                            gconstpointer key, guint key_len,
                                                            gpointer *value, guint *value_len);
FluxExtPatTreeCursor *flux_ext_pat_tree_traverse           (FluxExtPatTree *ext_pat_tree,
                                                            gboolean traverse_values);

void                  flux_ext_pat_tree_calc_node_sizes    (guint key_len, guint value_len,
                                                            guint *key_node_len, guint *value_node_len);

void                  flux_ext_pat_tree_cursor_init        (FluxExtPatTreeCursor *ext_pat_tree_cursor,
                                                            FluxExtPatTree *ext_pat_tree);
void                  flux_ext_pat_tree_cursor_finalize    (FluxObject *object);

gboolean              flux_ext_pat_tree_cursor_read        (FluxExtPatTreeCursor *cursor,
                                                            gpointer *key, guint *key_len,
                                                            gpointer *value, guint *value_len);
gboolean              flux_ext_pat_tree_cursor_get_randomized (FluxExtPatTreeCursor *cursor);
void                  flux_ext_pat_tree_cursor_set_randomized (FluxExtPatTreeCursor *cursor,
                                                               gboolean randomized);

#endif  /* FLUX_EXT_PAT_TREE_H */
