/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-file-io.h - File I/O channel.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_FILE_IO_H
#define _FLUX_FILE_IO_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-io.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-io.h>
#endif

#include <glib.h>

#define FLUX_FILE_IO(object) ((FluxFileIO *) object)

#define FLUX_FILE_IO_ERROR flux_file_io_error_quark ()

typedef enum
{
  FLUX_FILE_IO_ERROR_INVALID_FD,
  FLUX_FILE_IO_ERROR_DEVICE_IO,
  FLUX_FILE_IO_ERROR_NO_SPACE,
  FLUX_FILE_IO_ERROR_OTHER
}
FluxFileIOError;

typedef enum
{
  FLUX_FILE_IO_IDLE,
  FLUX_FILE_IO_READING,
  FLUX_FILE_IO_WRITING
}
FluxFileIOState;

struct _FluxFileIO
{
  FluxIO                    io;

  GMutex                   *mutex;
  GCond                    *wakeup_cond;
  GThread                  *worker_thread;
  guint                     running : 1;

  guint                     user_cb_id;

  FluxFileIOState           state;
  gint                      fd;
  guint64                   real_position;

  FluxEofFunc              *eof_func;
  gpointer                  eof_func_data;

  FluxEowFunc              *eow_func;
  gpointer                  eow_func_data;

  FluxErrorFunc            *error_func;
  gpointer                  error_func_data;
  GError                   *error;
};

FluxFileIO    *flux_file_io_new                       (gint fd);
void           flux_file_io_init                      (FluxFileIO *file_io, gint fd);
void           flux_file_io_finalize                  (FluxObject *file_io);

gint           flux_file_io_get_fd                    (FluxFileIO *file_io);
guint64        flux_file_io_get_size                  (FluxFileIO *file_io);
guint64        flux_file_io_get_position              (FluxFileIO *file_io);

void           flux_file_io_set_eof_func              (FluxFileIO *file_io, FluxEofFunc *func, gpointer data);
void           flux_file_io_set_eow_func              (FluxFileIO *file_io, FluxEowFunc *func, gpointer data);
void           flux_file_io_set_error_func            (FluxFileIO *file_io, FluxErrorFunc *func, gpointer data);

void           flux_file_io_seek_to                   (FluxFileIO *file_io, guint64 position);
void           flux_file_io_seek_relative             (FluxFileIO *file_io, gint64 offset);

GQuark         flux_file_io_error_quark               (void);

#endif /* _FLUX_FILE_IO_H */
