/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-file-util.c - Simple utilities for working with files.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <glib.h>

#ifndef G_PLATFORM_WIN32
# include <unistd.h>
#else
# include <io.h>
# include <direct.h>
# include "win32/win32-compat.h"
#endif

#include "flux-types.h"
#include "flux-file-util.h"

gchar *
flux_path_names_join (const gchar *first, const gchar *last)
{
  gchar temp_path [512];

  if (!strlen (first))
    return NULL;

  strcpy (temp_path, first);

  if (strlen (last))
  {
    if (*(temp_path + strlen (temp_path) - 1) != '/') strcat (temp_path, "/");

    if (*last == '/') strcat (temp_path, last + 1);
    else              strcat (temp_path, last);
  }
  
  return g_strdup (temp_path);
}

gboolean
flux_path_remove (const gchar *path)
{
  DIR *dir_d;
  struct dirent *entry;
  struct stat statbuf;
  char *path_this;

  dir_d = opendir (path);
  if (!dir_d)
    return FALSE;

  for ( ; (entry = readdir(dir_d)); )
  {
    if (!strcmp ("..", entry->d_name) || !strcmp(".", entry->d_name))
      continue;

    path_this = g_malloc (strlen (path) + strlen (entry->d_name) + 2);
    strcpy (path_this, path);
    strcat (path_this, "/");
    strcat (path_this, entry->d_name);

    stat (path_this, &statbuf);

#ifndef G_PLATFORM_WIN32
    if (S_ISDIR (statbuf.st_mode)) flux_path_remove (path_this);
    else                           remove           (path_this);
#else
    if (statbuf.st_mode & _S_IFDIR) flux_path_remove (path_this);
    else                            remove           (path_this);
#endif

    g_free (path_this);
  }

  closedir (dir_d);
  rmdir (path);
  return TRUE;
}
