/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-gobject-util.h - GObject utilities.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_GOBJECT_UTIL_H
#define _FLUX_GOBJECT_UTIL_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
#else
# include <flux/flux-types.h>
#endif

typedef struct
{
  gchar       *name;
  gchar       *nick;
  gchar       *blurb;
  GParamFlags  param_flags;
  guint        fundamental_type;
  GType        (*type_getter)(void);
  void         (*value_getter)(void);
  void         (*value_setter)(void);
}
FluxGObjectPropElem;

#define FLUX_GOBJECT_PROPERTY(name, nick, blurb, param_flags, fundamental_type, \
                              type_getter, value_getter, value_setter)          \
{ name, nick, blurb, param_flags, fundamental_type,                             \
  type_getter, (void (*)(void)) value_getter, (void (*)(void)) value_setter },

#define FLUX_GOBJECT_PROPERTIES_BEGIN(self_prefix) \
  static FluxGObjectPropElem self_prefix##_prop_list [] = { { NULL },

#define FLUX_GOBJECT_PROPERTIES_END() };

#define FLUX_GOBJECT_MAKE_IMPL(self_prefix, self_type, parent_type)               \
extern FluxGObjectPropElem self_prefix##_prop_list [];                            \
static GObjectClass *self_prefix##_parent_class;                                  \
                                                                                  \
/* User funcs; user will have to define these */                                  \
static void self_prefix##_class_init (self_type##Class *object_class);            \
static void self_prefix##_init (self_type *object);                               \
static void self_prefix##_dispose (self_type *object);                            \
static void self_prefix##_finalize (self_type *object);                           \
                                                                                  \
static void self_prefix##_simple_get_property (GObject *object, guint prop_id,    \
                                               GValue *value, GParamSpec *pspec)  \
{                                                                                 \
  flux_gobject_get_property (object, prop_id, value, pspec,                       \
                             self_prefix##_prop_list,                             \
                             G_N_ELEMENTS (self_prefix##_prop_list));             \
}                                                                                 \
                                                                                  \
static void self_prefix##_simple_set_property (GObject *object, guint prop_id,    \
                                               const GValue *value, GParamSpec *pspec)  \
{                                                                                 \
  flux_gobject_set_property (object, prop_id, value, pspec,                       \
                             self_prefix##_prop_list,                             \
                             G_N_ELEMENTS (self_prefix##_prop_list));             \
}                                                                                 \
                                                                                  \
static void self_prefix##_simple_dispose (GObject *object)                        \
{                                                                                 \
  self_prefix##_dispose ((self_type *) object);  /* User func */                  \
                                                                                  \
  if (G_OBJECT_CLASS (self_prefix##_parent_class)->dispose)                       \
    (* G_OBJECT_CLASS (self_prefix##_parent_class)->dispose) (object);            \
}                                                                                 \
                                                                                  \
static void self_prefix##_simple_finalize (GObject *object)                       \
{                                                                                 \
  self_prefix##_finalize ((self_type *) object);  /* User func */                 \
                                                                                  \
  if (G_OBJECT_CLASS (self_prefix##_parent_class)->finalize)                      \
    (* G_OBJECT_CLASS (self_prefix##_parent_class)->finalize) (object);           \
}                                                                                 \
                                                                                  \
static void self_prefix##_class_simple_init (GObjectClass *object_class)          \
{                                                                                 \
  self_prefix##_parent_class = g_type_class_ref (parent_type);                    \
  object_class->get_property = self_prefix##_simple_get_property;                 \
  object_class->set_property = self_prefix##_simple_set_property;                 \
  object_class->dispose = self_prefix##_simple_dispose;                           \
  object_class->finalize = self_prefix##_simple_finalize;                         \
  flux_gobject_class_install_properties (object_class, self_prefix##_prop_list,   \
                                         G_N_ELEMENTS (self_prefix##_prop_list)); \
  self_prefix##_class_init ((self_type##Class *) object_class);  /* User func */  \
}                                                                                 \
                                                                                  \
static void self_prefix##_simple_init (GObject *object)                           \
{                                                                                 \
  self_prefix##_init ((self_type *) object);  /* User func */                     \
}                                                                                 \
                                                                                  \
GType self_prefix##_get_type (void)                                               \
{                                                                                 \
  static GType type = 0;                                                          \
  if (!type)                                                                      \
  {                                                                               \
    static GTypeInfo const object_info =                                          \
    {                                                                             \
      sizeof (self_type##Class),                                                  \
                                                                                  \
      (GBaseInitFunc) NULL,                                                       \
      (GBaseFinalizeFunc) NULL,                                                   \
                                                                                  \
      (GClassInitFunc) self_prefix##_class_simple_init,                           \
      (GClassFinalizeFunc) NULL,                                                  \
      NULL,  /* class_data */                                                     \
                                                                                  \
      sizeof (self_type),                                                         \
      0,  /* n_preallocs */                                                       \
      (GInstanceInitFunc) self_prefix##_simple_init,                              \
    };                                                                            \
    type = g_type_register_static (parent_type, #self_type, &object_info, 0);     \
  }                                                                               \
  return type;                                                                    \
}

void flux_gobject_class_install_properties (GObjectClass *object_class, FluxGObjectPropElem *props,
                                            gint n_props);
void flux_gobject_get_property             (GObject *object, guint prop_id, GValue *value,
                                            GParamSpec *pspec, FluxGObjectPropElem *props, gint n_props);
void flux_gobject_set_property             (GObject *object, guint prop_id, const GValue *value,
                                            GParamSpec *pspec, FluxGObjectPropElem *props, gint n_props);

#endif  /* _FLUX_GOBJECT_UTIL_H */
