/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-info.c - Compiled-in information about the library.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include <stdlib.h>
#include <string.h>
#include "flux-info.h"

static gchar *_flux_unknown    = "unknown";

static gchar *_flux_package    = PACKAGE;
static gchar *_flux_version    = VERSION;
static gchar *_flux_build_host = HOSTNAME;

#ifdef PRINTABLE_OS_NAME
static gchar *_flux_build_os   = PRINTABLE_OS_NAME;
#else
static gchar *_flux_build_os   = _flux_unknown;
#endif

#ifdef PRINTABLE_CC_NAME
static gchar *_flux_build_cc   = PRINTABLE_CC_NAME;
#else
static gchar *_flux_build_cc   = _flux_unknown;
#endif

static gchar *_flux_build_cpu  = CPU;

const gchar *
flux_info_get_package (void)
{
  return _flux_package;
}

const gchar *
flux_info_get_version (void)
{
  return _flux_version;
}

const gchar *
flux_info_get_build_host (void)
{
  return _flux_build_host;
}

const gchar *
flux_info_get_build_os (void)
{
  return _flux_build_os;
}

const gchar *
flux_info_get_build_cc (void)
{
  return _flux_build_cc;
}

const gchar *
flux_info_get_build_c_features (void)
{
  static gchar *flux_build_c_extensions = NULL;
  
  if (!flux_build_c_extensions)
  {
    /* Build the extension string */
    
    flux_build_c_extensions = g_strjoin (" ",
#ifdef __STRICT_ANSI__
                                         "ansi",
#endif

#if defined(__cplusplus) || defined(c_plusplus)
                                         "c++",
#endif

#ifdef _POSIX_SOURCE
                                         "posix",
#endif

#ifdef _XOPEN_SOURCE_EXTENDED
                                         "xpg xopen",
#elif defined(_XOPEN_SOURCE)
                                         "xpg",
#endif

#ifdef _BSD_SOURCE
                                         "bsd",
#endif

#ifdef _SVID_SOURCE
                                         "svid",
#endif

#ifdef __GNUC__
                                         "gnu",
#endif

#if defined(WIN32) || defined (_WIN32) || defined (G_PLATFORM_WIN32)
                                         "win32",
#endif

#ifdef _MSC_VER
                                         "msvc",
#endif

#ifdef macintosh
                                         "macintosh",
#endif
                                         NULL);
  }

  return flux_build_c_extensions;
}

const gchar *
flux_info_get_build_cpu (void)
{
  if (!strlen (_flux_build_cpu))
    return _flux_unknown;

  return _flux_build_cpu;
}

const gchar *
flux_info_get_build_time (void)
{
  return NULL;
}
