/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-init.c - Subsystem initialization.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include <stdlib.h>
#include <string.h>
#include <gmp.h>
#include "flux-init.h"

#ifdef G_PLATFORM_WIN32
# include <winsock2.h>
#endif

static void *
my_malloc (size_t size)
{
  return g_malloc (size);
}

static void *
my_realloc (void *mem, size_t old_size, size_t new_size)
{
  return g_realloc (mem, new_size);
}

static void
my_free (void *mem, size_t size)
{
  g_free (mem);
}

gboolean
flux_init_mp (void)
{
  static GStaticMutex mutex          = G_STATIC_MUTEX_INIT;
  static gboolean     mp_initialized = FALSE;

  g_static_mutex_lock (&mutex);

  if (mp_initialized)
  {
    g_static_mutex_unlock (&mutex);
    return TRUE;
  }

  mp_set_memory_functions (my_malloc, my_realloc, my_free);
  mp_initialized = TRUE;

  g_static_mutex_unlock (&mutex);

  return TRUE;
}

gboolean
flux_init_sockets (void)
{
#ifdef G_PLATFORM_WIN32
  WORD    req_version;
  WSADATA wsa_data;
  gint    err;
  static  GStaticMutex mutex       = G_STATIC_MUTEX_INIT;
  static  gboolean     initialized = FALSE;

  g_static_mutex_lock (&mutex);

  if (initialized)
  {
    g_static_mutex_unlock (&mutex);
    return TRUE;
  }

  req_version = MAKEWORD (2, 0);
  err = WSAStartup (req_version, &wsa_data);
  if (err != 0)
  {
    g_static_mutex_unlock (&mutex);
    return FALSE;
  }

  if (LOBYTE (wsa_data.wVersion) != 2 ||
      HIBYTE (wsa_data.wVersion) != 0)
  {
    WSACleanup ();
    g_static_mutex_unlock (&mutex);
    return FALSE;
  }

  initialized = TRUE;

  g_static_mutex_unlock (&mutex);
#endif

  return TRUE;
}
