/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-io-group.h - Resource sharing for FluxIOs.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_IO_GROUP_H
#define _FLUX_IO_GROUP_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-io.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-io.h>
#endif

#include <glib.h>

#define FLUX_IO_GROUP(object) ((FluxIOGroup *) (object))

struct _FluxIOGroup
{
  FluxObject  object;

  gint        goal_cps_in;
  gint        goal_cps_out;

  gint        used_cps_in;
  gint        used_cps_out;

  GHashTable *member_table;
};

FluxIOGroup *flux_io_group_new                 (void);
void         flux_io_group_init                (FluxIOGroup *io_group);
void         flux_io_group_finalize            (FluxObject *object);

void         flux_io_group_add_group           (FluxIOGroup *io_group, FluxIOGroup *child_io_group);
void         flux_io_group_add_io              (FluxIOGroup *io_group, FluxIO *io);
gboolean     flux_io_group_remove_member       (FluxIOGroup *io_group, gpointer member);

void         flux_io_group_get_cps_goal        (FluxIOGroup *io_group, gint *cps_in, gint *cps_out);
void         flux_io_group_set_cps_goal        (FluxIOGroup *io_group, gint cps_in, gint cps_out);

void         flux_io_group_get_cps             (FluxIOGroup *io_group, gint *cps_in, gint *cps_out);

gboolean     flux_io_group_get_member_priority (FluxIOGroup *io_group, gconstpointer member,
                                                gdouble *priority_in, gdouble *priority_out);
gboolean     flux_io_group_set_member_priority (FluxIOGroup *io_group, gpointer member,
                                                gdouble priority_in, gdouble priority_out);

#endif /* _FLUX_IO_GROUP_H */
