/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-io.h - Transport-independent full-duplex stream class.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_IO_H
#define _FLUX_IO_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-cipher.h"
# include "flux-dual-fifo.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-cipher.h>
# include <flux/flux-dual-fifo.h>
#endif

#include <glib.h>

#define FLUX_IO(object) ((FluxIO *) object)

struct _FluxIO
{
  FluxDualFifo     dfifo;

  FluxCipher      *cipher;
  gint             read_compression_level;
  gint             write_compression_level;

  gint             read_cap,  read_slice_cap,  read_count,  read_cps;
  gint             write_cap, write_slice_cap, write_count, write_cps;

  FluxIOReadFunc  *read_func;
  gpointer         read_func_data;
  guint            read_threshold;

  FluxIOWriteFunc *write_func;
  gpointer         write_func_data;
  guint            write_threshold;
};

void     flux_io_init                (FluxIO *io);
void     flux_io_finalize            (FluxObject *io);

gboolean flux_io_is_connected        (FluxIO *io);

void     flux_io_set_cipher          (FluxIO *io, FluxCipher *cipher);

void     flux_io_set_read_func       (FluxIO *io, FluxIOReadFunc *func, gpointer data);
void     flux_io_set_write_func      (FluxIO *io, FluxIOWriteFunc *func, gpointer data);

void     flux_io_set_read_threshold  (FluxIO *io, guint threshold);
void     flux_io_set_write_threshold (FluxIO *io, guint threshold);

gint     flux_io_get_read_cps_goal   (FluxIO *io);
void     flux_io_set_read_cps_goal   (FluxIO *io, gint cps);

gint     flux_io_get_write_cps_goal  (FluxIO *io);
void     flux_io_set_write_cps_goal  (FluxIO *io, gint cps);

gint     flux_io_get_read_cps        (FluxIO *io);
gint     flux_io_get_write_cps       (FluxIO *io);

void     flux_io_set_compression     (FluxIO *io, gint read_level, gint write_level);

#endif /* _FLUX_IO_H */
