/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-ip-addr.h - Internet address.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_IP_ADDR_H
#define _FLUX_IP_ADDR_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#include <glib.h>

#ifndef G_PLATFORM_WIN32
# include <arpa/inet.h>
#else
# include <winsock2.h>
#endif

#define FLUX_IP_ADDR(object) ((FluxIPAddr *) object)

typedef enum
{
  FLUX_IP_ADDR_INVALID,
  FLUX_IP_ADDR_IPV4,
  FLUX_IP_ADDR_IPV6
}
FluxIPAddrType;

struct _FluxIPAddr
{
  FluxObject     object;

  FluxIPAddrType type;
  guint8         addr [16];
};

FluxIPAddr      *flux_ip_addr_new          (void);
void             flux_ip_addr_init         (FluxIPAddr *ip_addr);
void             flux_ip_addr_finalize     (FluxObject *ip_addr);

FluxIPAddrType   flux_ip_addr_get_type     (FluxIPAddr *ip_addr);
gboolean         flux_ip_addr_is_valid     (FluxIPAddr *ip_addr);

gchar           *flux_ip_addr_get_string   (FluxIPAddr *ip_addr);
gboolean         flux_ip_addr_set_string   (FluxIPAddr *ip_addr, const gchar *string);

gboolean         flux_ip_addr_get_raw      (FluxIPAddr *ip_addr, guint8 *dest, gint *len);
gboolean         flux_ip_addr_set_raw      (FluxIPAddr *ip_addr, const guint8 *src, gint len);

struct sockaddr *flux_ip_addr_get_sockaddr (FluxIPAddr *ip_addr, gint port);
gboolean         flux_ip_addr_set_sockaddr (FluxIPAddr *ip_addr, struct sockaddr *sa);

gboolean         flux_ip_addr_is_loopback  (FluxIPAddr *ip_addr);
gboolean         flux_ip_addr_is_multicast (FluxIPAddr *ip_addr);
gboolean         flux_ip_addr_is_broadcast (FluxIPAddr *ip_addr);
gboolean         flux_ip_addr_is_reserved  (FluxIPAddr *ip_addr);
gboolean         flux_ip_addr_is_private   (FluxIPAddr *ip_addr);
gboolean         flux_ip_addr_is_internet  (FluxIPAddr *ip_addr);

FluxIPAddr      *flux_ip_addr_find_internet_interface (void);
GList           *flux_ip_addr_get_interfaces          (void);
FluxIPAddr      *flux_ip_addr_get_interface_to        (FluxIPAddr *ip_addr);

#endif /* _FLUX_IP_ADDR_H */
