/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-ip-resolver.h - DNS resolver.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_IP_RESOLVER_H
#define _FLUX_IP_RESOLVER_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-ip-addr.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-ip-addr.h>
#endif

#include <glib.h>

#define FLUX_IP_RESOLVER(object) ((FluxIPResolver *) object)

struct _FluxIPResolver
{
  FluxObject   object;

  GMutex      *mutex;
  GThreadPool *thread_pool;
  GHashTable  *lookup_table;

  guint        next_lookup_id;
};

FluxIPResolver *flux_ip_resolver_new            (void);
gboolean        flux_ip_resolver_init           (FluxIPResolver *resolver);
void            flux_ip_resolver_finalize       (FluxObject *resolver);

FluxIPResolver *flux_ip_resolver_get_default    (void);

guint           flux_ip_resolver_lookup         (FluxIPResolver *resolver, const gchar *name,
                                                 FluxIPLookupFunc *func, gpointer data);
guint           flux_ip_resolver_lookup_reverse (FluxIPResolver *resolver, FluxIPAddr *addr,
                                                 FluxIPLookupFunc *func, gpointer data);

void            flux_ip_resolver_cancel_lookup  (FluxIPResolver *resolver, guint lookup_id);

#endif /* _FLUX_IP_RESOLVER_H */
