/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-marshaller.h - Platform-independent en/decoder for C structs.
 *                     Can also generate/parse human-readable data.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_MARSHALLER_H
#define _FLUX_MARSHALLER_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#define FLUX_MARSHALLER(obj) ((FluxMarshaller *) (obj))

/* Flags pertaining to a complete struct */

#define FLUX_MARSHALLER_CLEAR (1 << 0)

/* Struct entry types */

#define FLUX_MARSHAL_TYPED_INT(name, type, struct_type, member, substitute_default, default) \
  { name, type, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_default, GINT_TO_POINTER (default) }

#define FLUX_MARSHAL_TYPED_UINT(name, type, struct_type, member, substitute_default, default) \
  { name, type, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_default, GUINT_TO_POINTER (default) }

#define FLUX_MARSHAL_BOOLEAN(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_UINT (name, FLUX_MARSHAL_BOOLEAN_TYPE, struct_type, member, \
                           substitute_default, default)

#define FLUX_MARSHAL_INT8(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_INT (name, FLUX_MARSHAL_INT8_TYPE, struct_type, member, \
                          substitute_default, default)

#define FLUX_MARSHAL_UINT8(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_UINT (name, FLUX_MARSHAL_UINT8_TYPE, struct_type, member, \
                           substitute_default, default)

#define FLUX_MARSHAL_INT16(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_INT (name, FLUX_MARSHAL_INT16_TYPE, struct_type, member, \
                          substitute_default, default)

#define FLUX_MARSHAL_UINT16(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_UINT (name, FLUX_MARSHAL_UINT16_TYPE, struct_type, member, \
                           substitute_default, default)

#define FLUX_MARSHAL_INT32(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_INT (name, FLUX_MARSHAL_INT32_TYPE, struct_type, member, \
                          substitute_default, default)

#define FLUX_MARSHAL_UINT32(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_UINT (name, FLUX_MARSHAL_UINT32_TYPE, struct_type, member, \
                           substitute_default, default)

#define FLUX_MARSHAL_INT64(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_INT (name, FLUX_MARSHAL_INT64_TYPE, struct_type, member, \
                          substitute_default, default)

#define FLUX_MARSHAL_UINT64(name, struct_type, member, substitute_default, default) \
  FLUX_MARSHAL_TYPED_UINT (name, FLUX_MARSHAL_UINT64_TYPE, struct_type, member, \
                           substitute_default, default)

#define FLUX_MARSHAL_FLOAT32(name, struct_type, member, substitute_default, default) \
  { name, FLUX_MARSHAL_FLOAT32_TYPE, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_default, \
    FLUX_FLOAT_TO_POINTER (default) }

#define FLUX_MARSHAL_FLOAT64(name, struct_type, member, substitute_default, default) \
  { name, FLUX_MARSHAL_FLOAT32_TYPE, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_default, \
    FLUX_FLOAT_TO_POINTER (default) }

#define FLUX_MARSHAL_RAW_DYNAMIC(name, struct_type, member, meta_member, substitute_default, default) \
  { name, FLUX_MARSHAL_RAW_DYNAMIC_TYPE, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), G_STRUCT_OFFSET (struct_type, meta_member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, meta_member), substitute_default, (gpointer) default }

#define FLUX_MARSHAL_RAW_FIXED(name, struct_type, member, substitute_default, default) \
  { name, FLUX_MARSHAL_RAW_FIXED_TYPE, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_default, (gpointer) default }

#define FLUX_MARSHAL_STRING_DYNAMIC(name, struct_type, member, substitute_default, default) \
  { name, FLUX_MARSHAL_STRING_DYNAMIC_TYPE, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_default, (gpointer) default }

#define FLUX_MARSHAL_STRING_FIXED(name, struct_type, member, substitute_default, default) \
  { name, FLUX_MARSHAL_STRING_FIXED_TYPE, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_default, (gpointer) default }

#define FLUX_MARSHAL_STRUCT_DYNAMIC(name, struct_type, member, substitute_zero, sub_struct_name) \
  { name, FLUX_MARSHAL_STRUCT_DYNAMIC_TYPE, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_zero, (gpointer) sub_struct_name }

#define FLUX_MARSHAL_STRUCT_FIXED(name, struct_type, member, substitute_zero, sub_struct_name) \
  { name, FLUX_MARSHAL_STRUCT_FIXED_TYPE, G_STRUCT_OFFSET (struct_type, member), \
    FLUX_STRUCT_MEMBER_SIZE (struct_type, member), 0, 0, substitute_zero, (gpointer) sub_struct_name }

#define FLUX_MARSHAL_BRANCH_BEGIN(name) \
  { name, FLUX_MARSHAL_BRANCH_BEGIN_TYPE, -1, -1, -1, -1, FALSE, NULL }

#define FLUX_MARSHAL_BRANCH_END() \
  { "", FLUX_MARSHAL_BRANCH_END_TYPE, -1, -1, -1, -1, FALSE, NULL }

#define FLUX_MARSHAL_END() \
  { NULL, FLUX_MARSHAL_LAST_TYPE, -1, -1, -1, -1, FALSE, NULL }

struct _FluxMarshaller
{
  FluxObject          obj;

  GHashTable         *types;
};

struct _FluxMarshalledStruct
{
  gchar                     *name;
  gint                       size;
  guint                      flags;
  FluxMarshalledStructEntry *entries;
};

struct _FluxMarshalledStructEntry
{
  gchar                         *name;
  FluxMarshalledStructEntryType  type;
  glong                          offset;
  gint                           size;
  glong                          meta_offset;
  gint                           meta_size;
  gboolean                       substitute_default;
  gpointer                       default_value;
};

/* --- */

FluxMarshaller *flux_marshaller_new             (void);
void            flux_marshaller_init            (FluxMarshaller *marshaller);
void            flux_marshaller_finalize        (FluxObject *object);

gboolean        flux_marshaller_register        (FluxMarshaller *marshaller, gchar *struct_name,
                                                 gint struct_size, guint flags,
                                                 FluxMarshalledStructEntry *struct_entries);
gint            flux_marshaller_register_many   (FluxMarshaller *marshaller, FluxMarshalledStruct *structs);
gboolean        flux_marshaller_unregister      (FluxMarshaller *marshaller, const gchar *struct_name);
gint            flux_marshaller_unregister_many (FluxMarshaller *marshaller, FluxMarshalledStruct *structs);
gint            flux_marshaller_unregister_all  (FluxMarshaller *marshaller);

FluxTree       *flux_marshaller_encode_tree     (FluxMarshaller *marshaller, const gchar *struct_name,
                                                 gconstpointer struct_ptr);
gboolean        flux_marshaller_decode_tree     (FluxMarshaller *marshaller, FluxTree *tree,
                                                 gchar **struct_name, gpointer *struct_ptr);

gboolean        flux_marshaller_encode_pack     (FluxMarshaller *marshaller, const gchar *struct_name,
                                                 gconstpointer struct_ptr, gpointer *pack_ptr,
                                                 guint *pack_len);
gboolean        flux_marshaller_decode_pack     (FluxMarshaller *marshaller, const gchar *struct_name,
                                                 gconstpointer pack_ptr, guint pack_len,
                                                 gpointer *struct_ptr);
gboolean        flux_marshaller_free_struct     (FluxMarshaller *marshaller, const gchar *struct_name,
                                                 gpointer struct_ptr);

#endif  /* _FLUX_MARSHALLER_H */
