/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-mem-chunk.h - Low-overhead same-size memory block allocator.
 *
 * Copyright (C) 2000 Helix Code Inc.
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Michael Zucchi      <notzed@helixcode.com>
 *          Hans Petter Jansson <hpj@copyleft.no>
 *
 * Originally part of Evolution, Ximian's groupware client.
 */

#ifndef _FLUX_MEM_CHUNK_H
#define _FLUX_MEM_CHUNK_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-storage.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-storage.h>
#endif

#define FLUX_MEM_CHUNK(mem_chunk) ((FluxMemChunk *) mem_chunk)

typedef struct _MemChunkFreeNode MemChunkFreeNode;

struct _FluxMemChunk
{
  FluxStorage       storage;

  guint             blocksize; /* Number of atoms in a block */
  guint             atomsize;  /* Size of each atom */
  GPtrArray        *blocks;    /* Blocks of raw memory */
  MemChunkFreeNode *free;

  guint             size;
  GHashTable       *tag_table;
};

FluxMemChunk *flux_mem_chunk_new      (guint atomcount, guint atomsize);
void          flux_mem_chunk_init     (FluxMemChunk *mem_chunk, guint atomcount, guint atomsize);
void          flux_mem_chunk_finalize (FluxObject *mem_chunk);

gpointer      flux_mem_chunk_alloc    (FluxMemChunk *m);
gpointer      flux_mem_chunk_alloc0   (FluxMemChunk *m);
void          flux_mem_chunk_free     (FluxMemChunk *m, gpointer mem);

void          flux_mem_chunk_clean    (FluxMemChunk *m);
void          flux_mem_chunk_reset    (FluxMemChunk *m);

#endif  /* _FLUX_MEM_CHUNK_H */
