/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-object.h - Base class for Flux' naive but lightweight objects.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_OBJECT_H
#define _FLUX_OBJECT_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
#else
# include <flux/flux-types.h>
#endif

#define FLUX_OBJECT(obj) ((FluxObject *) (obj))

struct _FluxObject
{
  guint type;
  guint refs;
  FluxFinalizeFunc *finalize_func;
};

#ifdef FLUX_DISABLE_CHECKS
# define flux_object_check(object, type)
#else
# define flux_object_check(object, type) flux_object_check_real (object, type)
#endif

/* --- */

guint             flux_register_class           (const gchar *name, guint parent_type);
void              flux_unregister_class_by_name (const gchar *name);
void              flux_unregister_class_by_type (guint type);
const gchar      *flux_get_class_name           (guint type);
guint             flux_get_class_type           (const gchar *name);
guint             flux_get_class_parent         (guint type);

FluxObject       *flux_object_new               (void);
void              flux_object_init              (FluxObject *obj);
void              flux_object_destroy           (FluxObject *obj);

gint              flux_object_get_refs          (FluxObject *obj);
void              flux_object_ref               (FluxObject *obj);
void              flux_object_unref             (FluxObject *obj);

guint             flux_object_get_type          (FluxObject *obj);
void              flux_object_set_type          (FluxObject *obj, guint type);
gboolean          flux_object_is_type           (FluxObject *obj, guint type);

FluxFinalizeFunc *flux_object_get_finalize_func (FluxObject *obj);
void              flux_object_set_finalize_func (FluxObject *obj, FluxFinalizeFunc *func);

void              flux_object_assert            (FluxObject *obj);
void              flux_object_check_real        (FluxObject *obj, guint type);

#endif  /* _FLUX_OBJECT_H */
