/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-packed-bin-tree.h - Packed, static binary trees.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef FLUX_PACKED_BIN_TREE_H
#define FLUX_PACKED_BIN_TREE_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-storage.h"
# include "flux-bin-path.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-storage.h>
# include <flux/flux-bin-path.h>
#endif

#define FLUX_PACKED_BIN_TREE(object) ((FluxPackedBinTree *) (object))

struct _FluxPackedBinTree
{
  FluxObject     object;

  FluxStorage   *storage;
  guint          elem_len;
};

FluxPackedBinTree *flux_packed_bin_tree_new             (FluxStorage *storage, guint elem_len);
gboolean           flux_packed_bin_tree_init            (FluxPackedBinTree *bin_tree, FluxStorage *storage,
                                                         guint elem_len);
void               flux_packed_bin_tree_finalize        (FluxObject *bin_tree);

guint              flux_packed_bin_tree_get_elem_len    (FluxPackedBinTree *bin_tree);
gint               flux_packed_bin_tree_get_n_leaves    (FluxPackedBinTree *bin_tree, FluxItem tree);

FluxItem           flux_packed_bin_tree_create_tree     (FluxPackedBinTree *bin_tree, guint n_leaves);
void               flux_packed_bin_tree_delete_tree     (FluxPackedBinTree *bin_tree, FluxItem tree);

gint               flux_packed_bin_tree_path_to_index   (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         FluxBinPath *path);
gint               flux_packed_bin_tree_path_to_leaf    (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         FluxBinPath *path);
FluxBinPath       *flux_packed_bin_tree_index_to_path   (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         gint index);
gint               flux_packed_bin_tree_index_to_leaf   (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         gint index);
FluxBinPath       *flux_packed_bin_tree_leaf_to_path    (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         gint leaf);
gint               flux_packed_bin_tree_leaf_to_index   (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         gint leaf);

gconstpointer      flux_packed_bin_tree_peek            (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         gint index);
gboolean           flux_packed_bin_tree_read            (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         gint index, gpointer dest);
gboolean           flux_packed_bin_tree_write           (FluxPackedBinTree *bin_tree, FluxItem tree,
                                                         gint index, gconstpointer src);

#endif  /* FLUX_PACKED_BIN_TREE_H */
