/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-pat-tree.h -
 *
 * Copyright (C) 2002-2003 Hans Petter Jansson
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef FLUX_PAT_TREE_H
#define FLUX_PAT_TREE_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-db-header.h"
# include "flux-disk-chunk.h"
# include "flux-mem-chunk.h"
# include "flux-storage.h"
# include "flux-cursor.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-db-header.h>
# include <flux/flux-disk-chunk.h>
# include <flux/flux-mem-chunk.h>
# include <flux/flux-storage.h>
# include <flux/flux-cursor.h>
#endif

#define FLUX_PAT_TREE(object)        ((FluxPatTree *) (object))
#define FLUX_PAT_TREE_CURSOR(object) ((FluxPatTreeCursor *) (object))

struct _FluxPatTreeCursor
{
  FluxCursor   cursor;

  FluxPatTree *pat_tree;

  GArray      *index_stack;
  FluxItem     key_index;

  guint        randomize         : 1;
  guint        started           : 1;
  guint        index_invalidated : 1;  /* If a concurrent removal has invalidated key_index */

  gpointer     find_key;
  gpointer     find_mask;

  guint        key_len;
};

struct _FluxPatTree
{
  FluxObject     object;

  gchar         *id;
  FluxItem       default_root;

  /* For fixed-len storage only */
  guint          key_len;    /* Size of key data appended to each node */
  guint          value_len;  /* Size of value data appended to each node */
  guint          fixed_len;  /* Node + key + value */

  FluxStorage   *storage;
  FluxMemChunk  *mem_chunk;

  GList         *cursors;
};

FluxPatTree       *flux_pat_tree_new             (FluxStorage *storage, const gchar *id,
                                                  guint key_len, guint value_len);
gboolean           flux_pat_tree_init            (FluxPatTree *pat_tree, FluxStorage *storage,
                                                  const gchar *id, guint key_len, guint value_len);
void               flux_pat_tree_finalize        (FluxObject *pat_tree);

FluxItem           flux_pat_tree_create_root     (FluxPatTree *pat_tree);

gboolean           flux_pat_tree_insert          (FluxPatTree *pat_tree, FluxItem root,
                                                  gconstpointer key, guint key_len,
                                                  gconstpointer value, guint value_len);
gboolean           flux_pat_tree_replace_closest (FluxPatTree *pat_tree, FluxItem root,
                                                  gconstpointer key, guint key_len,
                                                  gconstpointer value, guint value_len);
gboolean           flux_pat_tree_find            (FluxPatTree *pat_tree, FluxItem root,
                                                  gconstpointer key, guint key_len,
                                                  gpointer *value, guint *value_len);
gboolean           flux_pat_tree_find_closest    (FluxPatTree *pat_tree, FluxItem root,
                                                  gconstpointer key, guint key_len,
                                                  gpointer *closest_key, guint *closest_key_len,
                                                  gpointer *closest_value, guint *closest_value_len);
FluxPatTreeCursor *flux_pat_tree_find_with_mask  (FluxPatTree *pat_tree, FluxItem root,
                                                  gconstpointer key, gconstpointer key_mask,
                                                  guint key_len);
gboolean           flux_pat_tree_remove          (FluxPatTree *pat_tree, FluxItem root,
                                                  gconstpointer key, guint key_len);

FluxPatTreeCursor *flux_pat_tree_traverse        (FluxPatTree *pat_tree, FluxItem root);
gboolean           flux_pat_tree_get_random      (FluxPatTree *pat_tree, FluxItem root,
                                                  gpointer *key, guint *key_len,
                                                  gpointer *value, guint *value_len);

guint              flux_pat_tree_calc_node_size  (guint key_len, guint value_len);

void               flux_pat_tree_cursor_init     (FluxPatTreeCursor *cursor, FluxPatTree *pat_tree,
                                                  FluxItem root, guint key_len);
void               flux_pat_tree_cursor_finalize (FluxObject *cursor);

gboolean           flux_pat_tree_cursor_read     (FluxPatTreeCursor *cursor,
                                                  gpointer *key, guint *key_len,
                                                  gpointer *value, guint *value_len);
gboolean           flux_pat_tree_cursor_get_randomized (FluxPatTreeCursor *cursor);
void               flux_pat_tree_cursor_set_randomized (FluxPatTreeCursor *cursor, gboolean randomized);

#endif  /* FLUX_PAT_TREE_H */
