/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-region.h - Virtual area for rectangle operations.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: ???
 *          Hans Petter Jansson <hpj@copyleft.no>
 *
 * Taken from GTK+ which took it from XFree (I think). Original authors
 * were not listed. The version I got didn't compile, and the only
 * modifications I've done were to make it compile and integrate better
 * with Flux.
 */

#ifndef _FLUX_REGION_H
#define _FLUX_REGION_H

#include <glib.h>

typedef struct _FluxRegion    FluxRegion;
typedef struct _FluxRectangle FluxRectangle;
typedef struct _FluxSegment   FluxSegment;
typedef struct _FluxSpan      FluxSpan;
typedef struct _FluxPoint     FluxPoint;

typedef FluxSegment FluxRegionBox;

struct _FluxRectangle
{
  gint x;
  gint y;
  gint width;
  gint height;
};

struct _FluxSegment
{
  gint x1;
  gint y1;
  gint x2;
  gint y2;
};

struct _FluxSpan
{
  gint x;
  gint y;
  gint width;
};

struct _FluxPoint
{
  gint x;
  gint y;
};

/* 
 *   clip region
 */

struct _FluxRegion
{
  long size;
  long numRects;
  FluxRegionBox *rects;
  FluxRegionBox extents;
};

/* GC fill rule for polygons
 *  EvenOddRule
 *  WindingRule
 */

typedef enum
{
  FLUX_EVEN_ODD_RULE,
  FLUX_WINDING_RULE
}
FluxFillRule;

/* Types of overlapping between a rectangle and a region
 * GDK_OVERLAP_RECTANGLE_IN: rectangle is in region
 * GDK_OVERLAP_RECTANGLE_OUT: rectangle in not in region
 * GDK_OVERLAP_RECTANGLE_PART: rectangle in partially in region
 */
typedef enum
{
  FLUX_OVERLAP_RECTANGLE_IN,
  FLUX_OVERLAP_RECTANGLE_OUT,
  FLUX_OVERLAP_RECTANGLE_PART
}
FluxOverlapType;

typedef void (*FluxSpanFunc) (FluxSpan *span,
			      gpointer data);

FluxRegion *flux_region_new       (void);

/* FluxRegion *flux_region_polygon   (FluxPoint     *points,
 *				 gint          npoints,
 *				 FluxFillRule   fill_rule);
 */

FluxRegion *flux_region_copy      (FluxRegion    *region);
FluxRegion *flux_region_rectangle (FluxRectangle *rectangle);
void       flux_region_destroy   (FluxRegion    *region);

void	       flux_region_get_clipbox    (FluxRegion     *region,
                                          FluxRectangle  *rectangle);
void           flux_region_get_rectangles (FluxRegion     *region,
                                          FluxRectangle **rectangles,
                                          gint          *n_rectangles);

gboolean       flux_region_empty    (FluxRegion    *region);
gboolean       flux_region_equal    (FluxRegion    *region1,
				    FluxRegion    *region2);
gboolean       flux_region_point_in (FluxRegion    *region,
				    int           x,
				    int           y);
FluxOverlapType flux_region_rect_in  (FluxRegion    *region,
				    FluxRectangle *rect);

void flux_region_offset          (FluxRegion    *region,
				 gint          dx,
				 gint          dy);
void flux_region_shrink          (FluxRegion    *region,
				 gint          dx,
				 gint          dy);
void flux_region_union_with_rect (FluxRegion    *region,
				 FluxRectangle *rect);
void flux_region_intersect       (FluxRegion    *source1,
				 FluxRegion    *source2);
void flux_region_union           (FluxRegion    *source1,
			 	 FluxRegion    *source2);
void flux_region_subtract        (FluxRegion    *source1,
				 FluxRegion    *source2);
void flux_region_xor             (FluxRegion    *source1,
				 FluxRegion    *source2);

void flux_region_spans_intersect_foreach (FluxRegion   *region,
					 FluxSpan     *spans,
					 int          n_spans,
					 gboolean     sorted,
					 FluxSpanFunc  function,
					 gpointer     data);

#endif /* _FLUX_REGION_H */
