/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-rmd160.h - RIPE-MD160 hash function.
 *
 * Copyright (C) 1998 Free Software Foundation, Inc.
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: GNUPG team
 *          Hans Petter Jansson <hpj@copyleft.no>
 *
 * Taken from GNUPG and adapted to Flux semantics.
 */

#ifndef _FLUX_RMD160_H
#define _FLUX_RMD160_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#include <glib.h>

#define FLUX_RMD160(object) ((FluxRmd160 *) object)

#define FLUX_RMD160_HASH_LEN 20

struct _FluxRmd160
{
  FluxObject object;

  guint32 h0, h1, h2, h3, h4;
  guint32 nblocks;
  guchar  buf [64];
  gint    count;
};

FluxRmd160 *flux_rmd160_new         (void);
void        flux_rmd160_init        (FluxRmd160 *rmd160);
void        flux_rmd160_finalize    (FluxRmd160 *rmd160);

void        flux_rmd160_begin       (FluxRmd160 *rmd160);
void        flux_rmd160_end         (FluxRmd160 *rmd160);

void        flux_rmd160_add         (FluxRmd160 *rmd160, gconstpointer in, guint len);
void        flux_rmd160_get_hash    (FluxRmd160 *rmd160, gpointer out);

void        flux_rmd160_hash_single (gpointer out, gconstpointer in, guint len);

#endif /* _FLUX_RMD160_H */
