/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-rsa.h - The RSA asymmetric cipher.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson.
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_RSA_H
#define _FLUX_RSA_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#include <gmp.h>

#define FLUX_RSA_KEY(key)        ((FluxRSAKey *) key)
#define FLUX_RSA_PUBLIC_KEY(key) ((FluxRSAPublicKey *) key)
#define FLUX_RSA_SECRET_KEY(key) ((FluxRSASecretKey *) key)

struct _FluxRSAKey
{
  FluxObject object;

  mpz_t n;
};

struct _FluxRSAPublicKey
{
  FluxRSAKey key;

  mpz_t e;
};

struct _FluxRSASecretKey
{
  FluxRSAKey key;

  mpz_t d;
};

/* --- */

gboolean          flux_rsa_generate_keypair    (guint key_size, FluxRSAPublicKey **public_key,
                                                FluxRSASecretKey **secret_key);
FluxRSAKey       *flux_rsa_dup_key             (FluxRSAKey *key);

void              flux_rsa_finalize_public_key (FluxRSAPublicKey *pkey);
void              flux_rsa_finalize_secret_key (FluxRSASecretKey *skey);

FluxRSAPublicKey *flux_rsa_import_public_key   (const guchar *in, guint in_len);
FluxRSASecretKey *flux_rsa_import_secret_key   (const guchar *in, guint in_len);

void              flux_rsa_export_public_key   (FluxRSAPublicKey *public_key, guchar **out, guint *out_len);
void              flux_rsa_export_secret_key   (FluxRSASecretKey *secret_key, guchar **out, guint *out_len);

guint             flux_rsa_get_key_length      (FluxRSAKey *key);
guint             flux_rsa_get_block_size      (FluxRSAKey *key);

void              flux_rsa_encrypt             (FluxRSAPublicKey *pkey,
                                                guchar      **ciphertext, guint *ciphertext_len,
                                                const guchar *plaintext,  guint  plaintext_len);
void              flux_rsa_decrypt             (FluxRSASecretKey *skey,
                                                guchar      **plaintext,  guint *plaintext_len,
                                                const guchar *ciphertext, guint  ciphertext_len);
void              flux_rsa_sign                (FluxRSASecretKey *skey,
                                                guchar      **signature,  guint *signature_len,
                                                const guchar *hash,       guint  hash_len);
gboolean          flux_rsa_verify              (FluxRSAPublicKey *pkey,
                                                const guchar *signature,  guint  signature_len,
                                                const guchar *hash,       guint  hash_len);

#endif  /* _FLUX_RSA_H */
