/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-socket-util.h - Socket utilities.
 *
 * Copyright (C) 2004 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_SOCKET_UTIL_H
#define _FLUX_SOCKET_UTIL_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-ip-addr.h"
#else
# include <flux/flux-types.h>
# include <flux-ip-addr.h>
#endif

#ifndef G_PLATFORM_WIN32
# include <arpa/inet.h>
#else
# include <winsock2.h>
#endif

#ifdef G_PLATFORM_WIN32
# define flux_socket_close(fd) closesocket (fd)
#else
# define flux_socket_close(fd) close (fd)
#endif

struct sockaddr *flux_sockaddr_new              (FluxIPAddrType type,
                                                 const gchar *addr_str, gint port);

gchar           *flux_sockaddr_to_string        (struct sockaddr *sa);

gint             flux_sockaddr_get_family       (struct sockaddr *sa);
gint             flux_sockaddr_get_len          (struct sockaddr *sa);
gpointer         flux_sockaddr_get_addr         (struct sockaddr *sa);
gint             flux_sockaddr_get_addr_len     (struct sockaddr *sa);
gint             flux_sockaddr_get_port         (struct sockaddr *sa);

gboolean         flux_sockaddr_is_ipv4          (struct sockaddr *sa);
gboolean         flux_sockaddr_is_ipv6          (struct sockaddr *sa);
gboolean         flux_sockaddr_is_ip            (struct sockaddr *sa);

gboolean         flux_addr_family_is_ipv4       (gint in_family);
gboolean         flux_addr_family_is_ipv6       (gint in_family);
gboolean         flux_addr_family_is_ip         (gint in_family);

gboolean         flux_socket_set_nonblock       (gint fd, gboolean nonblock);

gboolean         flux_wakeup_pipe_create        (gint *fd);
void             flux_wakeup_pipe_close         (gint *fd);
gint             flux_wakeup_pipe_get_watch_fd  (gint *fd);
void             flux_wakeup_pipe_wakeup        (gint *fd);
void             flux_wakeup_pipe_handle_wakeup (gint *fd);

#endif  /* _FLUX_SOCKET_UTIL_H */
