/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-storage.c - Abstract class for storage backends.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include "flux-storage.h"

#define assert_type(o) flux_object_check (FLUX_OBJECT (o), FLUX_STORAGE_TYPE)

void
flux_storage_init (FluxStorage *storage)
{
  flux_object_init (FLUX_OBJECT (storage));
  flux_object_set_type (FLUX_OBJECT (storage), FLUX_STORAGE_TYPE);
  flux_object_set_finalize_func (FLUX_OBJECT (storage), flux_storage_finalize);

  storage->capabilities = 0;
}

void
flux_storage_finalize (FluxObject *object)
{
  /* Nothing to do */
}

guint
flux_storage_get_capabilities (FluxStorage *storage)
{
  assert_type (storage);

  return storage->capabilities;
}

guint
flux_storage_get_arena_size (FluxStorage *storage)
{
  return storage->get_arena_size (storage);
}

guint
flux_storage_get_used_size (FluxStorage *storage)
{
  return storage->get_used_size (storage);
}

FluxItem
flux_storage_get_root (FluxStorage *storage, const gchar *tag)
{
  return storage->get_root (storage, tag);
}

gboolean
flux_storage_set_root (FluxStorage *storage, const gchar *tag, FluxItem item)
{
  return storage->set_root (storage, tag, item);
}

FluxItem
flux_storage_alloc (FluxStorage *storage, guint len)
{
  return storage->alloc (storage, len);
}

FluxItem
flux_storage_alloc0 (FluxStorage *storage, guint len)
{
  return storage->alloc0 (storage, len);
}

FluxItem
flux_storage_realloc (FluxStorage *storage, FluxItem item, guint len)
{
  return storage->realloc (storage, item, len);
}

gboolean
flux_storage_free (FluxStorage *storage, FluxItem item)
{
  return storage->free (storage, item);
}

gconstpointer
flux_storage_peek (FluxStorage *storage, FluxItem item,
                   guint offset, guint len)
{
  return storage->peek (storage, item, offset, len);
}

gboolean
flux_storage_read (FluxStorage *storage, FluxItem item, gpointer dest)
{
  return storage->do_read (storage, item, dest);
}

gboolean
flux_storage_read_full (FluxStorage *storage, FluxItem item, gpointer dest,
                        guint offset, guint len)
{
  return storage->do_read_full (storage, item, dest, offset, len);
}

gboolean
flux_storage_write (FluxStorage *storage, FluxItem item, gconstpointer src)
{
  return storage->do_write (storage, item, src);
}

gboolean
flux_storage_write_full (FluxStorage *storage, FluxItem item,
                         gconstpointer src, guint offset, guint len)
{
  return storage->do_write_full (storage, item, src, offset, len);
}

guint
flux_storage_get_item_len (FluxStorage *storage, FluxItem item)
{
  return storage->get_item_len (storage, item);
}
