/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-storage.h - Abstract class for storage backends.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_STORAGE_H
#define _FLUX_STORAGE_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
#endif

#include <glib.h>

#define FLUX_STORAGE(object) ((FluxStorage *) object)
#define FLUX_ITEM(item)      ((FluxItem) item)

#if GLIB_SIZEOF_VOID_P == 4

typedef guint32 FluxItem;
#define FLUX_ITEM_NULL ((FluxItem) 0xffffffffUL)

#else /* GLIB_SIZEOF_VOID_P == 8 */

typedef guint64 FluxItem;
#define FLUX_ITEM_NULL ((FluxItem) G_GINT64_CONSTANT (0xffffffffffffffffU))

#endif

/* For use with e.g. GHashTable */
#define FLUX_ITEM_TO_POINTER(item) ((gpointer) (item))
#define FLUX_POINTER_TO_ITEM(ptr)  ((FluxItem) (ptr))

typedef enum
{
  FLUX_STORAGE_VARIABLE_LENGTH = (1 << 0),
  FLUX_STORAGE_PERSISTENT      = (1 << 1)
}
FluxStorageCapability;

struct _FluxStorage
{
  FluxObject    object;

  guint         capabilities;

  guint         (*get_arena_size) (FluxStorage *storage);
  guint         (*get_used_size)  (FluxStorage *storage);

  FluxItem      (*get_root)       (FluxStorage *storage, const gchar *tag);
  gboolean      (*set_root)       (FluxStorage *storage, const gchar *tag, FluxItem item);

  FluxItem      (*alloc)          (FluxStorage *storage, guint len);
  FluxItem      (*alloc0)         (FluxStorage *storage, guint len);
  FluxItem      (*realloc)        (FluxStorage *storage, FluxItem item, guint len);
  gboolean      (*free)           (FluxStorage *storage, FluxItem item);

  gconstpointer (*peek)           (FluxStorage *storage, FluxItem item,
                                   guint offset, guint len);
  gboolean      (*do_read)        (FluxStorage *storage, FluxItem item, gpointer dest);
  gboolean      (*do_read_full)   (FluxStorage *storage, FluxItem item, gpointer dest,
                                   guint offset, guint len);
  gboolean      (*do_write)       (FluxStorage *storage, FluxItem item, gconstpointer src);
  gboolean      (*do_write_full)  (FluxStorage *storage, FluxItem item,
                                   gconstpointer src, guint offset, guint len);

  guint         (*get_item_len)   (FluxStorage *storage, FluxItem item);
};

void          flux_storage_init             (FluxStorage *storage);
void          flux_storage_finalize         (FluxObject *object);

guint         flux_storage_get_capabilities (FluxStorage *storage);
guint         flux_storage_get_arena_size   (FluxStorage *storage);
guint         flux_storage_get_used_size    (FluxStorage *storage);

FluxItem      flux_storage_get_root         (FluxStorage *storage, const gchar *tag);
gboolean      flux_storage_set_root         (FluxStorage *storage, const gchar *tag, FluxItem item);

FluxItem      flux_storage_alloc            (FluxStorage *storage, guint len);
FluxItem      flux_storage_alloc0           (FluxStorage *storage, guint len);
FluxItem      flux_storage_realloc          (FluxStorage *storage, FluxItem item, guint len);
gboolean      flux_storage_free             (FluxStorage *storage, FluxItem item);

gconstpointer flux_storage_peek             (FluxStorage *storage, FluxItem item,
                                             guint offset, guint len);
gboolean      flux_storage_read             (FluxStorage *storage, FluxItem item, gpointer dest);
gboolean      flux_storage_read_full        (FluxStorage *storage, FluxItem item, gpointer dest,
                                             guint offset, guint len);
gboolean      flux_storage_write            (FluxStorage *storage, FluxItem item, gconstpointer src);
gboolean      flux_storage_write_full       (FluxStorage *storage, FluxItem item,
                                             gconstpointer src, guint offset, guint len);

guint         flux_storage_get_item_len     (FluxStorage *storage, FluxItem item);

#endif /* _FLUX_STORAGE_H */
