/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-string-util.c - Simple string utilities.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include "flux-types.h"
#include "flux-string-util.h"

gint
flux_memcasecmp (const guchar *p0, const guchar *p1, guint len)
{
  for (; len; len--)
  {
    if (g_ascii_tolower (*(p0++)) != g_ascii_tolower (*(p1++)))
      break;
  }
  
  if (len) return (g_ascii_tolower (*(p0 - 1)) - g_ascii_tolower (*(p1 - 1)));
  return 0;
}

gint
flux_memcspn (const guchar *s, guint len, const guchar *reject)
{
  gint i;

  for (i = 0; i < len; i++)
    if (strchr (reject, (char) *((char *) s + i)))
      break;

  return i;
}

void 
flux_memlwr (guchar *s, guint len)
{
  for (; len; len--, s++)
    *((guchar *) s) = g_ascii_tolower (*((guchar *) s));
}

guchar *
flux_memrchr (const guchar *s, guchar c, gint n)
{
  guchar *p;

  if (!n) return NULL;

  for (p = (guchar *) s + n - 1; p >= (guchar *) s; p--)
    if (*p == (guchar) c)
      break;

  if (p >= (guchar *) s)
    return (gpointer) p;

  return NULL;
}

gint 
flux_memspn (const guchar *s, guint len, const guchar *accept)
{
  gint i;

  for (i = 0; i < len; i++)
    if (!strchr (accept, (char) *((char *) s + i)))
      break;

  return i;
}

void
flux_memupr (guchar *s, guint len)
{
  for (; len; len--, s++)
    *((guchar *) s) = g_ascii_toupper (*((guchar *) s));
}
