/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-tcp-io.h - TCP I/O channel.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_TCP_IO_H
#define _FLUX_TCP_IO_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-io.h"
# include "flux-cipher.h"
# include "flux-ip-addr.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-io.h>
# include <flux/flux-cipher.h>
# include <flux/flux-ip-addr.h>
#endif

#include <glib.h>

#define FLUX_TCP_IO(object) ((FluxTcpIO *) object)

struct _FluxTcpIO
{
  FluxIO                    io;

  FluxTcpIOState            state;
  FluxCipher               *cipher;

  gint                      fd;

  guint                     callback_id;
  guint                     lookup_id;

  FluxIPAddr               *remote_ip_addr;
  gint                      remote_port;

  /* Timestamp used for connect/disconnect timeout */
  guint                     timestamp;

  /* Size of the cipher element we're currently waiting for. If zero,
   * the size itself is currently being read. */
  guint16                   cipher_element_size;

  guint                     user_connected_flag         : 1,  /* Backend -> user */
                            user_disconnected_flag      : 1,  /* Backend -> user */
                            disconnect_request_flag     : 1,  /* User -> backend */
                            have_connected_fd           : 1,  /* If our fd is connected */
                            reading_cipher_invitation   : 1,
                            reading_cipher_confirmation : 1;

  FluxTcpLinkFunc          *link_func;
  gpointer                  link_func_data;
};

FluxTcpIO      *flux_tcp_io_new                        (void);
FluxTcpIO      *flux_tcp_io_new_with_fd                (gint fd);
FluxTcpIO      *flux_tcp_io_new_with_cipher            (FluxCipher *cipher);
FluxTcpIO      *flux_tcp_io_new_with_fd_and_cipher     (gint fd, FluxCipher *cipher);
void            flux_tcp_io_init                       (FluxTcpIO *tcp_io, gint fd,
                                                        FluxCipher *cipher, gint read_compression_level,
                                                        gint write_compression_level);
void            flux_tcp_io_finalize                   (FluxObject *tcp_io);

void            flux_tcp_io_connect_begin              (FluxTcpIO *tcp_io, const gchar *host, guint port);
void            flux_tcp_io_disconnect                 (FluxTcpIO *tcp_io);

void            flux_tcp_io_set_link_func              (FluxTcpIO *tcp_io, FluxTcpLinkFunc *func, gpointer data);

FluxCipher     *flux_tcp_io_get_cipher                 (FluxTcpIO *tcp_io);
void            flux_tcp_io_set_cipher                 (FluxTcpIO *tcp_io, FluxCipher *cipher);

gint            flux_tcp_io_get_fd                     (FluxTcpIO *tcp_io);
FluxTcpIOState  flux_tcp_io_get_state                  (FluxTcpIO *tcp_io);

FluxIPAddr     *flux_tcp_io_get_local_ip_addr          (FluxTcpIO *tcp_io);
gint            flux_tcp_io_get_local_port             (FluxTcpIO *tcp_io);

FluxIPAddr     *flux_tcp_io_get_remote_ip_addr         (FluxTcpIO *tcp_io);
gint            flux_tcp_io_get_remote_port            (FluxTcpIO *tcp_io);

#endif /* _FLUX_TCP_IO_H */
