/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-tcp-listener.h - Listener for TCP connections.
 *
 * Copyright (C) 2001-2004 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_TCP_LISTENER_H
#define _FLUX_TCP_LISTENER_H

#ifdef __FLUX_BUILD
# include "flux-types.h"
# include "flux-object.h"
# include "flux-cipher.h"
# include "flux-tcp-io.h"
#else
# include <flux/flux-types.h>
# include <flux/flux-object.h>
# include <flux/flux-cipher.h>
# include <flux/flux-tcp-io.h>
#endif

#include <glib.h>

#define FLUX_TCP_LISTENER(object) ((FluxTcpListener *) object)

struct _FluxTcpListener
{
  FluxObject             object;

  gint                   port;
  gint                   fd;
  gint                   read_compression_level;
  gint                   write_compression_level;
  FluxCipher            *cipher;

  GHashTable            *wait_connect_table;

  guint                  callback_id;

  FluxTcpConnectionFunc *connection_func;
  gpointer               connection_func_data;

  guint                  spawned_object_size;
};

gboolean         flux_tcp_listener_init                (FluxTcpListener *tl, gint port, FluxCipher *cipher);
FluxTcpListener *flux_tcp_listener_new                 (gint port);
FluxTcpListener *flux_tcp_listener_new_with_cipher     (gint port, FluxCipher *cipher);
void             flux_tcp_listener_finalize            (FluxObject *tl);

void             flux_tcp_listener_get_compression     (FluxTcpListener *tl, gint *read_level, gint *write_level);
void             flux_tcp_listener_set_compression     (FluxTcpListener *tl, gint read_level, gint write_level);

void             flux_tcp_listener_set_cipher          (FluxTcpListener *tl, FluxCipher *cipher);
void             flux_tcp_listener_set_connection_func (FluxTcpListener *tl, FluxTcpConnectionFunc *func,
                                                        gpointer data);

gint             flux_tcp_listener_get_port            (FluxTcpListener *tl);

#endif /* _FLUX_TCP_LISTENER_H */
