/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-types.h - Global types and definitions.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson.
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#ifndef _FLUX_TYPES_H
#define _FLUX_TYPES_H

#include <glib.h>
#include <glib-object.h>

/* --- Macros - doesn't really belong here --- */

/* Lame, but enables user to define small float constants with some precision
 * and store them in pointers */
#define FLUX_FLOAT_TO_POINTER(f) (GINT_TO_POINTER ((gint) ((f) * 1000.0)))
#define FLUX_POINTER_TO_FLOAT(p) (GPOINTER_TO_INT (p) / 1000.0)

#define FLUX_STRUCT_MEMBER_SIZE(struct_type, member) \
  ((gint) (sizeof (((struct_type*) 0)->member)))

/* --- Objects --- */

typedef enum
{
  FLUX_INVALID_TYPE,

  FLUX_OBJECT_TYPE,
  FLUX_BOOL_ARRAY_TYPE,
  FLUX_BIN_PATH_TYPE,
  FLUX_EXT_HASH_TABLE_TYPE,
  FLUX_DATA_CACHE_TYPE,
  FLUX_CACHED_FILE_TYPE,
  FLUX_RMD160_TYPE,
  FLUX_RNG_TYPE,
  FLUX_TWOFISH_KEY_TYPE,
  FLUX_ELGAMAL_KEY_TYPE,
  FLUX_ELGAMAL_PUBLIC_KEY_TYPE,
  FLUX_ELGAMAL_SECRET_KEY_TYPE,
  FLUX_RSA_KEY_TYPE,
  FLUX_RSA_PUBLIC_KEY_TYPE,
  FLUX_RSA_SECRET_KEY_TYPE,
  FLUX_EC_KEY_TYPE,
  FLUX_EC_PUBLIC_KEY_TYPE,
  FLUX_EC_SECRET_KEY_TYPE,
  FLUX_CIPHER_TYPE,
  FLUX_IP_ADDR_TYPE,
  FLUX_IP_RESOLVER_TYPE,
  FLUX_FIFO_TYPE,
  FLUX_DUAL_FIFO_TYPE,
  FLUX_IO_TYPE,
  FLUX_FILE_IO_TYPE,
  FLUX_TCP_IO_TYPE,
  FLUX_TCP_LISTENER_TYPE,
  FLUX_IO_GROUP_TYPE,
  FLUX_CURSOR_TYPE,
  FLUX_COMM_TYPE,
  FLUX_ENVOY_TYPE,
  FLUX_MARSHALLER_TYPE,
  FLUX_STORAGE_TYPE,
  FLUX_MEM_CHUNK_TYPE,
  FLUX_MEM_HEAP_TYPE,
  FLUX_DISK_CHUNK_TYPE,
  FLUX_DISK_HEAP_TYPE,
  FLUX_DISK_HEAP_CURSOR_TYPE,
  FLUX_PACKED_BIN_TREE_TYPE,
  FLUX_PAT_TREE_TYPE,
  FLUX_PAT_TREE_CURSOR_TYPE,
  FLUX_EXT_PAT_TREE_TYPE,
  FLUX_EXT_PAT_TREE_CURSOR_TYPE,
  FLUX_INTERVAL_TYPE,

  FLUX_LAST_TYPE,
  FLUX_FREE_TYPE = 0x2e2e2e2e
}
FluxObjectType;

/* --- Typedefs --- */

typedef struct _FluxObject                FluxObject;
typedef struct _FluxBoolArray             FluxBoolArray;
typedef struct _FluxBinPath               FluxBinPath;
typedef struct _FluxExtHashTable          FluxExtHashTable;
typedef struct _FluxDataCache             FluxDataCache;
typedef struct _FluxCachedFile            FluxCachedFile;
typedef struct _FluxRmd160                FluxRmd160;
typedef struct _FluxRng                   FluxRng;
typedef struct _FluxTwofishKey            FluxTwofishKey;
typedef struct _FluxElgamalKey            FluxElgamalKey;
typedef struct _FluxElgamalPublicKey      FluxElgamalPublicKey;
typedef struct _FluxElgamalSecretKey      FluxElgamalSecretKey;
typedef struct _FluxRSAKey                FluxRSAKey;
typedef struct _FluxRSAPublicKey          FluxRSAPublicKey;
typedef struct _FluxRSASecretKey          FluxRSASecretKey;
typedef struct _FluxECKey                 FluxECKey;
typedef struct _FluxECPublicKey           FluxECPublicKey;
typedef struct _FluxECSecretKey           FluxECSecretKey;
typedef struct _FluxCipher                FluxCipher;
typedef struct _FluxIPAddr                FluxIPAddr;
typedef struct _FluxIPResolver            FluxIPResolver;
typedef struct _FluxFifo                  FluxFifo;
typedef struct _FluxDualFifo              FluxDualFifo;
typedef struct _FluxIO                    FluxIO;
typedef struct _FluxFileIO                FluxFileIO;
typedef struct _FluxTcpIO                 FluxTcpIO;
typedef struct _FluxTcpListener           FluxTcpListener;
typedef struct _FluxIOGroup               FluxIOGroup;
typedef struct _FluxCursor                FluxCursor;
typedef struct _FluxComm                  FluxComm;
typedef struct _FluxEnvoy                 FluxEnvoy;
typedef struct _FluxEnvoyDequeuer         FluxEnvoyDequeuer;
typedef struct _FluxDB                    FluxDB;
typedef struct _FluxTree                  FluxTree;
typedef struct _FluxMarshaller            FluxMarshaller;
typedef struct _FluxMarshalledStruct      FluxMarshalledStruct;
typedef struct _FluxMarshalledStructEntry FluxMarshalledStructEntry;
typedef struct _FluxStorage               FluxStorage;
typedef struct _FluxMemChunk              FluxMemChunk;
typedef struct _FluxMemHeap               FluxMemHeap;
typedef struct _FluxDiskChunk             FluxDiskChunk;
typedef struct _FluxDiskHeap              FluxDiskHeap;
typedef struct _FluxDiskHeapCursor        FluxDiskHeapCursor;
typedef struct _FluxPackedBinTree         FluxPackedBinTree;
typedef struct _FluxPatTree               FluxPatTree;
typedef struct _FluxPatTreeCursor         FluxPatTreeCursor;
typedef struct _FluxExtPatTree            FluxExtPatTree;
typedef struct _FluxExtPatTreeCursor      FluxExtPatTreeCursor;
typedef struct _FluxInterval              FluxInterval;

/* --- Enums --- */

typedef enum
{
  FLUX_PRINT_COMPACT,
  FLUX_PRINT_KNR,
  FLUX_PRINT_ALLMAN
}
FluxPrintMode;

typedef enum
{
  FLUX_TCP_IO_STATE_CONNECTING,
  FLUX_TCP_IO_STATE_CONNECTED,
  FLUX_TCP_IO_STATE_DISCONNECTING,
  FLUX_TCP_IO_STATE_DISCONNECTED,
}
FluxTcpIOState;

typedef enum
{
  FLUX_TCP_CONNECTION_ESTABLISHED,
  FLUX_TCP_CONNECTION_FAILED,
  FLUX_TCP_CONNECTION_CLOSED
}
FluxTcpLinkMessage;

typedef enum
{
  FLUX_TREE_ERROR_NONE,
  FLUX_TREE_ERROR_TOO_MANY_CLOSE_BRACES
}
FluxTreeError;

typedef enum
{
  FLUX_COMM_ORDER_DEPTH_FIRST,
  FLUX_COMM_ORDER_BREADTH_FIRST
}
FluxCommOrder;

typedef enum
{
  FLUX_MARSHAL_BOOLEAN_TYPE,
  FLUX_MARSHAL_INT8_TYPE,
  FLUX_MARSHAL_UINT8_TYPE,
  FLUX_MARSHAL_INT16_TYPE,
  FLUX_MARSHAL_UINT16_TYPE,
  FLUX_MARSHAL_INT32_TYPE,
  FLUX_MARSHAL_UINT32_TYPE,
  FLUX_MARSHAL_INT64_TYPE,
  FLUX_MARSHAL_UINT64_TYPE,
  FLUX_MARSHAL_FLOAT32_TYPE,
  FLUX_MARSHAL_FLOAT64_TYPE,

  FLUX_MARSHAL_RAW_DYNAMIC_TYPE,
  FLUX_MARSHAL_RAW_FIXED_TYPE,

  FLUX_MARSHAL_STRING_DYNAMIC_TYPE,
  FLUX_MARSHAL_STRING_FIXED_TYPE,

  FLUX_MARSHAL_STRUCT_DYNAMIC_TYPE,
  FLUX_MARSHAL_STRUCT_FIXED_TYPE,

  FLUX_MARSHAL_BRANCH_BEGIN_TYPE,
  FLUX_MARSHAL_BRANCH_END_TYPE,

  FLUX_MARSHAL_LAST_TYPE
}
FluxMarshalledStructEntryType;

/* --- Callback types --- */

typedef void (FluxFinalizeFunc)        (FluxObject *);
typedef gint (FluxByteIteratorFunc)    (gchar *data, gint len, void *user_data);
typedef void (FluxIPLookupFunc)        (GList *addr_list, GList *name_list, gpointer data);
typedef void (FluxTcpConnectionFunc)   (FluxTcpListener *tl, FluxTcpIO *tio, gpointer data);
typedef void (FluxTcpLinkFunc)         (FluxTcpIO *tio, FluxTcpLinkMessage message, gpointer data);
typedef void (FluxIOReadFunc)          (FluxIO *io, guint min_len, gpointer data);
typedef void (FluxIOWriteFunc)         (FluxIO *io, guint min_len, gpointer data);
typedef void (FluxEofFunc)             (FluxFileIO *file_io, guint len_left, gpointer data);
typedef void (FluxEowFunc)             (FluxFileIO *file_io, gpointer data);
typedef void (FluxErrorFunc)           (FluxObject *object, GError *error, gpointer data);
typedef void (FluxCommDequeueFunc)     (FluxComm *comm, FluxTree *tree, gpointer data);
typedef void (FluxCommSentFunc)        (FluxComm *comm, FluxTree *tree, gpointer data);
typedef gboolean (FluxEnvoyDequeueFunc) (FluxEnvoy *envoy, FluxTree *tree, gpointer data);
typedef void (FluxEnvoySentFunc)       (FluxEnvoy *envoy, FluxTree *tree, gpointer data);
typedef void (FluxReadHook)            (gpointer data);
typedef void (FluxWriteHook)           (gpointer data);
typedef gint (FluxStreamReaderFunc)    (gpointer source, guchar *block, guint pos, guint size);
typedef gint (FluxStreamProcessorFunc) (guchar *block, guint pos, guint size, gpointer data);
typedef gint (FluxStreamDualProcessorFunc) (guchar *block1, guchar *block2, guint pos, guint size, gpointer data);

#endif  /* _FLUX_TYPES_H */
