/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* flux-util.c - Simple utilities.
 *
 * Copyright (C) 2001-2003 Hans Petter Jansson
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#include "config.h"
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include "flux-types.h"
#include "flux-util.h"

gint
flux_compress_uint (guint n, guint8 *dest)
{
  gint len = 0;

  if (dest)
  {
    dest [len] = n & 0x7f;

    for (n >>= 7; n; n >>= 7)
    {
      dest [len] |= 0x80;
      len++;
      dest [len] = n & 0x7f;
    }
  }
  else
  {
    for (n >>= 7; n; n >>= 7)
      len++;
  }

  return len + 1;
}

gint
flux_decompress_uint (guint *n, const guint8 *src, gint max_len)
{
  guint m     = 0;
  gint  len   = 0;
  gint  shift = 0;

  if (max_len < 0)
    max_len = 5;
  else if (max_len == 0)
    return 0;

  for (;;)
  {
    m |= ((src [len] & 0x7f) << shift) & (0x7f << shift);

    /* End of sequence? */
    if (!(src [len] & 0x80))
      break;

    len++;

    /* End of buffer? This is an error condition. */
    if (len == max_len)
      return -1;

    shift += 7;
  }

  *n = m;
  return len + 1;
}
