#include "config.h"
#include "flux-disk-heap.h"
#include <string.h>     /* memset, memcpy */
#include <sys/types.h>  /* open */
#include <sys/stat.h>   /* open */
#include <fcntl.h>      /* open */

#define NUM_STRINGS    10000
#define MAX_STRING_LEN 256
#define ITERATIONS     4000000

extern void flux_disk_heap_dump (FluxDiskHeap *disk_heap);

static gchar
int_to_hex (int n)
{
  if (n < 0 || n > 0x0f)
    g_error ("Integer not within base16 digit range.");

  if (n < 0x0a)
    return ('0' + n);

  return ('a' + n - 0x0a);
}

static void
dump_hex (guchar *buf, guint len)
{
  gint i;

  for (i = 0; i < len; i++)
    g_print ("%c%c ", int_to_hex (buf [i] >> 4), int_to_hex (buf [i] & 0x0f));

  g_print ("\n");
}

static void
stress (FluxStorage *storage)
{
  gchar           *str_tab [NUM_STRINGS];
  FluxItem  ptr_tab [NUM_STRINGS];
  gint             i;

  for (i = 0; i < NUM_STRINGS; i++)
  {
    str_tab [i] = NULL;
    ptr_tab [i] = FLUX_ITEM_NULL;
  }

  for (i = 0; i < ITERATIONS; i++)
  {
    gint index = g_random_int_range (0, NUM_STRINGS);

    if (str_tab [index] == NULL)
    {
      gint len;

      len = g_random_int_range (2, MAX_STRING_LEN);
      str_tab [index] = g_malloc (len);
      memset (str_tab [index], 'a', len);
      str_tab [index] [len - 1] = '\0';

      ptr_tab [index] = flux_storage_alloc (storage, len - 1);
      flux_storage_write (storage, ptr_tab [index], str_tab [index]);
    }
    else
    {
      gchar *str;
      gint   len;

      len = flux_storage_get_item_len (storage, ptr_tab [index]);
      str = g_malloc (len + 1);
      str [len] = '\0';
      flux_storage_read (storage, ptr_tab [index], str);

      if (len != strlen (str_tab [index]))
	g_error ("Length mismatch! Expected %d, got %d.", strlen (str_tab [index]), len);

      if (memcmp (str, str_tab [index], len))
      {
	dump_hex (str_tab [index], len);
	dump_hex (str, len);
	g_error ("Record changed! (at byte %d)", strspn (str, "a"));
      }

      g_free (str);

      memset (str_tab [index], 0, len);
      flux_storage_write (storage, ptr_tab [index], str_tab [index]);

      flux_storage_free (storage, ptr_tab [index]);
      g_free (str_tab [index]);

      ptr_tab [index] = FLUX_ITEM_NULL;
      str_tab [index] = NULL;
    }
  }

  g_print ("Passed stress test!\n\n");

#if 0
  flux_disk_heap_dump (storage);
#endif

#if 0
  g_print ("File size: %d  Free size: %d\n", flux_storage_get_file_size (storage),
	   flux_storage_get_free_size (storage));
#endif
}

gint
main (gint argc, gchar *argv [])
{
  FluxStorage *storage;
  gint         fd;

#if 1
  fd = open ("disk-heap.db", O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
  storage = FLUX_STORAGE (flux_disk_heap_new (fd));
#else
  storage = FLUX_STORAGE (flux_mem_heap_new ());
#endif

  g_random_set_seed (time (NULL));

  if (!strcmp (argv [1], "stress"))
  {
    stress (storage);
  }

  flux_object_unref (FLUX_OBJECT (storage));
  return 0;
}
