/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* ip-addr-test.c - Tests for Internet address parsing.
 *
 * Copyright (C) 2004 Hans Petter Jansson.
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#define __FLUX_BUILD

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "flux-types.h"
#include "flux-ip-addr.h"

static void
list_interfaces (void)
{
  FluxIPAddr *inet_if;
  GList      *list, *l;

  list = flux_ip_addr_get_interfaces ();

  for (l = list; l; l = g_list_next (l))
  {
    FluxIPAddr *ip_addr = l->data;
    gchar      *ip_str;

    ip_str = flux_ip_addr_get_string (ip_addr);
    g_print ("Interface: %s\n", ip_str);
  }

  inet_if = flux_ip_addr_find_internet_interface ();
  if (inet_if)
    g_print ("Internet interface: %s\n", flux_ip_addr_get_string (inet_if));
  else
    g_print ("No internet interface found.\n");

  /* TODO: Unref FluxIPAddrs */
  g_list_free (list);
}

int
main (int argc, char *argv [])
{
  FluxIPAddr *ip_addr;

  if (argc < 2)
  {
    g_print ("Usage: %s <addr>\n", argv [0]);
    return 1;
  }

  /* List interfaces */

  list_interfaces ();

  /* Parse the provided address */

  ip_addr = flux_ip_addr_new ();

  if (flux_ip_addr_set_string (ip_addr, argv [1]))
  {
    g_print ("Parsed: %s\n", flux_ip_addr_get_string (ip_addr));
  }
  else
  {
    g_print ("Address didn't parse.\n");
  }

  return 0;
}
