/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

/* ip-addr-test.c - Tests for Internet address parsing.
 *
 * Copyright (C) 2004 Hans Petter Jansson.
 *
 * Flux is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flux is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * Authors: Hans Petter Jansson <hpj@copyleft.no>
 */

#define __FLUX_BUILD

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "flux-types.h"
#include "flux-ip-resolver.h"

static gboolean
dot_cb (gpointer data)
{
  g_print (".");
  fflush (stdout);

  return TRUE;
}

static void
lookup_cb (GList *addr_list, GList *name_list, gpointer data)
{
  GMainLoop *loop = data;
  GList     *l;

  g_print (" %d found.\n", g_list_length (addr_list));

  for (l = addr_list; l; l = g_list_next (l))
  {
    FluxIPAddr *ip_addr = l->data;

    g_print ("  %s\n", flux_ip_addr_get_string (ip_addr));
  }

  g_main_quit (loop);
}

static void
lookup_reverse_cb (GList *addr_list, GList *name_list, gpointer data)
{
  GMainLoop *loop = data;
  GList     *l;

  g_print (" %d found.\n", g_list_length (name_list));

  for (l = name_list; l; l = g_list_next (l))
  {
    gchar *name = l->data;

    g_print ("  %s\n", name);
  }

  g_main_quit (loop);
}

int
main (int argc, char *argv [])
{
  FluxIPAddr     *ip_addr;
  FluxIPResolver *ip_resolver;
  GMainLoop      *loop;
  gboolean        reverse;

  g_thread_init (NULL);

  if (argc < 2)
  {
    g_print ("Usage: %s <hostname|ip>\n", argv [0]);
    return 1;
  }

  ip_addr = flux_ip_addr_new ();
  flux_ip_addr_set_string (ip_addr, argv [1]);
  if (flux_ip_addr_is_valid (ip_addr))
    reverse = TRUE;
  else
    reverse = FALSE;

  if (reverse)
    g_print ("Looking up names for '%s' (%s).", argv [1],
             flux_ip_addr_get_type (ip_addr) == FLUX_IP_ADDR_IPV4 ? "IPv4" : "IPv6");
  else
    g_print ("Looking up IP addresses for '%s'.", argv [1]);

  fflush (stdout);

  loop = g_main_new (FALSE);

  g_timeout_add (500, (GSourceFunc) dot_cb, NULL);

  ip_resolver = flux_ip_resolver_new ();

  if (reverse)
    flux_ip_resolver_lookup_reverse (ip_resolver, ip_addr, lookup_reverse_cb, loop);
  else
    flux_ip_resolver_lookup (ip_resolver, argv [1], lookup_cb, loop);

  g_main_run (loop);

  flux_object_unref (FLUX_OBJECT (ip_addr));
  return 0;
}
