/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

#define __FLUX_BUILD

#include <stdio.h>
#include <string.h>
#include "flux-types.h"
#include "flux-tree.h"
#include "flux-marshaller.h"

#define ITERATIONS 1000000

typedef struct
{
  gchar *string_0;
  gchar *string_1;
}
StringBox;

typedef struct
{
  gint       int_0;
  guint      int_1;
  gchar     *string_0;
  gdouble    float_0;

  StringBox  strings;
}
Toplevel;

FluxMarshalledStructEntry toplevel_entries [] =
{
  FLUX_MARSHAL_BRANCH_BEGIN   ("integers"),
  FLUX_MARSHAL_INT32          ("int_0",    Toplevel, int_0, TRUE, 0),
  FLUX_MARSHAL_UINT32         ("int_1",    Toplevel, int_1, TRUE, 0),
  FLUX_MARSHAL_BRANCH_END     (),

  FLUX_MARSHAL_BRANCH_BEGIN   ("fixed_structs"),
  FLUX_MARSHAL_STRUCT_FIXED   ("sb",       Toplevel, strings, TRUE, "string_box"),
  FLUX_MARSHAL_BRANCH_END     (),

  FLUX_MARSHAL_BRANCH_BEGIN   ("strings & stuff"),
  FLUX_MARSHAL_STRING_DYNAMIC ("string_0", Toplevel, string_0, TRUE, "Faan"),
  FLUX_MARSHAL_FLOAT64        ("float_0",  Toplevel, float_0,  TRUE, 3.14),
  FLUX_MARSHAL_BRANCH_END     (),

  FLUX_MARSHAL_END            ()
};

FluxMarshalledStructEntry string_box_entries [] =
{
  FLUX_MARSHAL_STRING_DYNAMIC ("string_0", StringBox, string_0, TRUE, "Nummer 0"),
  FLUX_MARSHAL_STRING_DYNAMIC ("string_1", StringBox, string_1, TRUE, "Nummer 1"),

  FLUX_MARSHAL_END            ()
};

FluxMarshalledStruct marshal_info [] =
{
  /* Name         Size                Flags  Entries            */
  { "toplevel",   sizeof (Toplevel),  0,     toplevel_entries   },
  { "string_box", sizeof (StringBox), 0,     string_box_entries },
  { NULL }
};

int
main (int argc, char *argv [])
{
  FluxMarshaller *marshaller;
  Toplevel        toplevel;
  Toplevel       *post_toplevel;
  gpointer        pack;
  guint           pack_len;
  gint            i;

  /* Setup */

  marshaller = flux_marshaller_new ();

  flux_marshaller_register_many (marshaller, marshal_info);

  memset (&toplevel, 0, sizeof (toplevel));
  toplevel.int_0    = -42;
  toplevel.int_1    = 42;
  toplevel.string_0 = g_strdup ("Hallvete!");
  toplevel.float_0  = -441.114;

  toplevel.strings.string_0 = g_strdup ("Jepp.");

  /* Run */

  for (i = 0; i < ITERATIONS; i++)
  {
    flux_marshaller_encode_pack (marshaller, "toplevel", &toplevel, &pack, &pack_len);
    flux_marshaller_decode_pack (marshaller, "toplevel", pack, pack_len,
                                 (gpointer *) &post_toplevel);
    g_free (pack);
    g_free (post_toplevel->string_0);
    g_free (post_toplevel);
  }

  flux_object_unref (FLUX_OBJECT (marshaller));
  return 0;
}
