/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

#define __FLUX_BUILD

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "flux-types.h"
#include "flux-rmd160.h"
#include "flux-file-io.h"

#define BUF_SIZE 1024

typedef struct
{
  GMainLoop  *loop;
  FluxFileIO *file_io;
  FluxRmd160 *rmd160;
}
Context;

static gchar
int_to_hex (int n)
{
  if (n < 0 || n > 0x0f)
    g_error ("Integer not within base16 digit range.");

  if (n < 0x0a)
    return ('0' + n);

  return ('a' + n - 0x0a);
}

static gboolean
timeout_cb (gpointer data)
{
  Context *context = data;

  g_print ("\r[%3u%% done]", (guint) ((flux_file_io_get_position (context->file_io) * 100) /
           flux_file_io_get_size (context->file_io)));
  fflush (stdout);
  return TRUE;
}

static void
read_cb (FluxIO *io, guint min_len, gpointer data)
{
  Context *context = data;
  gchar   *buf;

  buf = g_malloc (min_len);
  flux_dual_fifo_read (FLUX_DUAL_FIFO (io), buf, min_len);
  flux_rmd160_add (context->rmd160, buf, min_len);
  g_free (buf);
}

static void
eof_cb (FluxFileIO *file_io, guint len_left, gpointer data)
{
  Context *context = data;
  gchar   *buf;

  buf = g_malloc (len_left);
  flux_dual_fifo_read (FLUX_DUAL_FIFO (file_io), buf, len_left);
  flux_rmd160_add (context->rmd160, buf, len_left);
  g_free (buf);

  g_main_quit (context->loop);
}

int
main (int argc, char *argv [])
{
  Context     *context;
  gint         fd;
  guchar       hash [20];
  gint         i;

  fd = open (argv [1], O_RDONLY);
  if (fd < 0)
    exit (1);

  context = g_new0 (Context, 1);

  context->loop = g_main_new (FALSE);
  g_timeout_add (1000, timeout_cb, context);

  context->rmd160 = flux_rmd160_new ();
  flux_rmd160_begin (context->rmd160);

  context->file_io = flux_file_io_new (fd);
  flux_io_set_read_threshold (FLUX_IO (context->file_io), 1024);
  flux_io_set_read_func      (FLUX_IO (context->file_io), read_cb, context);
  flux_file_io_set_eof_func  (context->file_io, eof_cb, context);

  g_main_run (context->loop);

  flux_rmd160_end (context->rmd160);
  flux_rmd160_get_hash (context->rmd160, hash);

  g_print ("\r");

  for (i = 0; i < 20; i++)
  {
    guchar b;

    b = hash [i];
    fputc (int_to_hex (b >> 4), stdout);
    fputc (int_to_hex (b & 0x0f), stdout);
  }

  g_print ("\n");

  flux_object_unref (FLUX_OBJECT (context->file_io));
  flux_object_unref (FLUX_OBJECT (context->rmd160));
  close (fd);
  return 0;
}
