/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

#define __FLUX_BUILD

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "flux-types.h"
#include "flux-rmd160.h"

#define BUF_SIZE 1024

static gchar
int_to_hex (int n)
{
  if (n < 0 || n > 0x0f)
    g_error ("Integer not within base16 digit range.");

  if (n < 0x0a)
    return ('0' + n);

  return ('a' + n - 0x0a);
}

int
main (int argc, char *argv [])
{
  FluxRmd160 *rmd160;
  gint         fd;
  gint         len;
  guchar       buf [BUF_SIZE];
  guchar       hash [20];
  gint         i;

  fd = open (argv [1], O_RDONLY);
  if (fd < 0)
    exit (1);

  rmd160 = flux_rmd160_new ();
  flux_rmd160_begin (rmd160);

  while ((len = read (fd, buf, BUF_SIZE)) > 0)
  {
    flux_rmd160_add (rmd160, buf, len);
  }

  flux_rmd160_end (rmd160);
  flux_rmd160_get_hash (rmd160, hash);

  for (i = 0; i < 20; i++)
  {
    guchar b;

    b = hash [i];
    fputc (int_to_hex (b >> 4), stdout);
    fputc (int_to_hex (b & 0x0f), stdout);
  }

  g_print ("\n");

  flux_object_unref (FLUX_OBJECT (rmd160));
  return 0;
}
