/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

#define __FLUX_BUILD

#include <stdio.h>
#include "flux-types.h"
#include "flux-tcp-io.h"

static gboolean
timeout_cb (gpointer data)
{
  FluxTcpIO *tio;

  tio = data;
  flux_tcp_io_connect_begin (tio, "127.0.0.1", 6005);
  return FALSE;
}

static void
link_cb (FluxTcpIO *tio, FluxTcpLinkMessage message, gpointer data)
{
  if (message == FLUX_TCP_CONNECTION_ESTABLISHED)
  {
    g_print ("Connected to server.\n");
  }
  else if (message == FLUX_TCP_CONNECTION_FAILED)
  {
    g_print ("Connection failed. Retrying.\n");
    g_timeout_add (1000, timeout_cb, tio);
  }
  else if (message == FLUX_TCP_CONNECTION_CLOSED)
  {
    g_print ("Server disconnected. Trying to reconnect.\n");
    g_timeout_add (1000, timeout_cb, tio);
  }
}

static void
read_cb (FluxIO *io, guint min_len, gpointer data)
{
  gchar buf [5];

  flux_dual_fifo_read (FLUX_DUAL_FIFO (io), buf, 5);
  fwrite (buf, sizeof (gchar), 5, stdout);
}


int
main (int argc, char *argv [])
{
  FluxTcpIO *tio;
  FluxCipher *cipher;
  GMainLoop *loop;

  cipher = flux_cipher_new ();
  flux_cipher_generate (cipher);

  tio = flux_tcp_io_new_with_cipher (cipher);

  flux_tcp_io_set_link_func  (tio, link_cb, NULL);
  flux_io_set_read_threshold (FLUX_IO (tio), 5);
  flux_io_set_read_func      (FLUX_IO (tio), read_cb, NULL);

  flux_tcp_io_connect_begin  (tio, "localhost", 6005);

  loop = g_main_new (FALSE);
  g_main_run (loop);

  return 0;
}
