/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

#define __FLUX_BUILD

#include <stdio.h>
#include "flux-types.h"
#include "flux-tcp-io.h"
#include "flux-comm.h"

FluxComm *comm;

static gboolean
timeout_cb (gpointer data)
{
  FluxTcpIO *tio;

  tio = data;
  flux_tcp_io_connect_begin (tio, "127.0.0.1", 6005);
  return FALSE;
}

static void
dequeue_cb (FluxComm *comm, FluxTree *tree, gpointer data)
{
  flux_tree_print_to_file (tree, stdout, FLUX_PRINT_ALLMAN);
  flux_tree_destroy (tree);
}

static void
link_cb (FluxTcpIO *tio, FluxTcpLinkMessage message, gpointer data)
{
  if (message == FLUX_TCP_CONNECTION_ESTABLISHED)
  {
    g_print ("Connected to server.\n");
  }
  else if (message == FLUX_TCP_CONNECTION_FAILED)
  {
    g_print ("Connection failed. Retrying.\n");
    g_timeout_add (1000, timeout_cb, tio);
  }
  else if (message == FLUX_TCP_CONNECTION_CLOSED)
  {
    g_print ("Server disconnected. Trying to reconnect.\n");
    g_timeout_add (1000, timeout_cb, tio);
  }
}

int
main (int argc, char *argv [])
{
  FluxTcpIO *tio;
  FluxCipher *cipher;
  GMainLoop *loop;

  cipher = flux_cipher_new ();
  flux_cipher_generate (cipher);

  tio = flux_tcp_io_new_with_cipher (cipher);
  flux_io_set_compression (FLUX_IO (tio), 9, 9);

  comm = flux_comm_new (FLUX_IO (tio));
  flux_comm_set_dequeue_func (comm, dequeue_cb, NULL);

  loop = g_main_new (FALSE);

  flux_tcp_io_set_link_func  (tio, link_cb, NULL);
  flux_tcp_io_connect_begin  (tio, "127.0.0.1", 6005);

  g_main_run (loop);

  return 0;
}
