/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */

#define __FLUX_BUILD

#include "flux-types.h"
#include "flux-tcp-listener.h"
#include "flux-tcp-io.h"
#include "flux-tree.h"
#include "flux-comm.h"

FluxComm *comm;

static gboolean
timeout_cb (gpointer data)
{
  FluxTree *top, *tree;

  top = flux_tree_new ();
  tree = flux_tree_new_with_parent_and_data (top, "Pekk", 4);
  flux_tree_new_with_parent_and_data (tree, "Pekk0", 5);
  flux_tree_new_with_parent_and_data (tree, "Pekk1", 5);
  flux_tree_new_with_parent_and_data (tree, "Pekk2", 5);

  g_print ("Server: Queuing tree.\n");

  flux_comm_enqueue (comm, top);
  return TRUE;
}

static void
link_cb (FluxTcpIO *tio, FluxTcpLinkMessage message, gpointer data)
{
  guint timeout_id;

  if (message == FLUX_TCP_CONNECTION_CLOSED)
  {
    timeout_id = GPOINTER_TO_UINT (g_dataset_get_data (tio, "timer"));
    g_dataset_remove_data (tio, "timer");
    g_source_remove (timeout_id);

    g_print ("Client disconnected.\n");
    flux_object_unref (FLUX_OBJECT (comm));
  }
}

static void
connection_cb (FluxTcpListener *tl, FluxTcpIO *tio, gpointer data)
{
  guint timeout_id;

  g_print ("Got connection callback (app).\n");
  flux_tcp_io_set_link_func (tio, link_cb, NULL);
  flux_io_set_compression (FLUX_IO (tio), 9, 9);

  comm = flux_comm_new (FLUX_IO (tio));

  timeout_id = g_timeout_add (1000, timeout_cb, tio);
  g_dataset_set_data (tio, "timer", GUINT_TO_POINTER (timeout_id));
}

int
main (int argc, char *argv [])
{
  FluxTcpListener *tl;
  FluxCipher *cipher;
  GMainLoop *loop;

  cipher = flux_cipher_new ();
  flux_cipher_generate (cipher);

  tl = flux_tcp_listener_new_with_cipher (6005, cipher);
  if (!tl)
    g_error ("Couldn't get server port.");

  flux_tcp_listener_set_connection_func (tl, connection_cb, NULL);

  loop = g_main_new (FALSE);
  g_main_run (loop);

  return 0;
}
